<?php
// +----------------------------------------------------------------------
// | OneKeyAdmin [ Believe that you can do better ]
// +----------------------------------------------------------------------
// | Copyright (c) 2020-2023 http://onekeyadmin.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: MUKE <513038996@qq.com>
// +----------------------------------------------------------------------
namespace app\admin\controller;

use think\facade\View;
use think\exception\ValidateException;
use app\admin\BaseController;
use app\admin\model\Config;
use app\admin\model\UserGroup;
use app\admin\model\User as UserModel;
use app\admin\validate\User as UserValidate;
/**
 * 用户管理
 */
class User extends BaseController
{
    /**
     * 显示资源列表
     */
    public function index()
    {
        if ($this->request->isPost()) {
            $input  = input('post.');
            $search = ['keyword','date','status'];
            $append = ['url'];
            $page     = empty($input['page']) ? 1 : $input['page'];
            $pageSize = empty($input['pageSize']) ? 20 : $input['pageSize'];
            if (! empty($input['prop']) && ! empty($input['order'])) {
                $order = [$input['prop'] => $input['order']];
            } else {
                $order = ['create_time' => 'desc'];
            }
            $count  = UserModel::withSearch($search, $input)->count();
            $data   = UserModel::withSearch($search, $input)->append($append)->with(['group'])->order($order)->page($page, $pageSize)->select();
            return json(['status' => 'success', 'message' => '获取成功', 'data' => $data, 'count' => $count]);
        } else {
            $config = Config::getVal('register');
            if (! $config) {
                $config = ['nickname' => '未命名', 'describe' => '这个人很懒，什么都没有留下。', 'cover' => ''];
            }
            $group = UserGroup::where('status', 1)->order('integral', 'asc')->select();
            View::assign(['group' => $group, 'config' => $config]);
            return View::fetch();
        }
    }

    /**
     * 保存新建的资源
     */
    public function save()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('save')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }   
            if (! empty($input['mobile'])) {
                if (UserModel::where('mobile', $input['mobile'])->value('id')) {
                    return json(['status' => 'error', 'message' => '手机号已经存在！']);
                }
            }
            if (! empty($input['email'])) {
                if (UserModel::where('email', $input['email'])->value('id')) {
                    return json(['status' => 'error', 'message' => '邮箱号已经存在！']);
                }
            }
            $date = date('Y-m-d H:i:s');
            $input['pay_paasword']     = '';
            $input['history_integral'] = $input['now_integral'];
            $input['login_ip']         = '';
            $input['login_count']      = 0;
            $input['birthday']         = $input['birthday'] ? $input['birthday'] : date('Y-m-d');
            $input['login_time']       = $date;
            $input['update_time']      = $date;
            $input['create_time']      = $date;
            $input['hide']             = 1;
            UserModel::create($input);
            return json(['status' => 'success', 'message' => '新增成功']);
        }
    }
    
    /**
     * 保存更新的资源
     */
    public function update()
    {
       if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('update')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }   
            $save = UserModel::find($input['id']);
            $save->nickname         = $input['nickname'];
            $save->sex              = $input['sex'];
            $save->cover            = $input['cover'];
            $save->describe         = $input['describe'];
            $save->birthday         = $input['birthday'];
            $save->create_time      = $input['create_time'];
            $save->reason           = $input['reason'];
            $save->save();
            return json(['status' => 'success', 'message' => '修改成功']);
       }
    }
    
    /**
     * 积分充值
     */
    public function integral()
    {
       if ($this->request->isPost()) {
            $input = input('post.');
            $save = UserModel::find($input['id']);
            $save->now_integral     = $save->now_integral + $input['integralTo'];
            $save->history_integral = $save->history_integral + $input['integralTo'];
            $save->save();
            return json(['status' => 'success', 'message' => '修改成功']);
       }
    }
    
    /**
     * 余额充值
     */
    public function balance()
    {
       if ($this->request->isPost()) {
            $input = input('post.');
            $save = UserModel::find($input['id']);
            $save->balance = $save->balance + $input['balanceTo'];
            $save->save();
            return json(['status' => 'success', 'message' => '修改成功']);
       }
    }
    
    /**
     * 邮箱号修改
     */
    public function email()
    {
       if ($this->request->isPost()) {
            $input = input('post.');
            $where[] = ['id', '<>', $input['id']];
            if (! empty($input['email'])) {
                if (UserModel::where('email', $input['email'])->where($where)->value('id')) {
                    return json(['status' => 'error', 'message' => '邮箱号已经存在！']);
                }
            }
            $save = UserModel::find($input['id']);
            $save->email = $input['email'];
            $save->save();
            return json(['status' => 'success', 'message' => '修改成功']);
       }
    }
    
    /**
     * 手机号修改
     */
    public function mobile()
    {
       if ($this->request->isPost()) {
            $input = input('post.');
            $where[] = ['id', '<>', $input['id']];
            if (! empty($input['mobile'])) {
                if (UserModel::where('mobile', $input['mobile'])->where($where)->value('id')) {
                    return json(['status' => 'error', 'message' => '手机号已经存在！']);
                }
            }
            $save = UserModel::find($input['id']);
            $save->mobile = $input['mobile'];
            $save->save();
            return json(['status' => 'success', 'message' => '修改成功']);
       }
    }
    
    /**
     * 密码修改
     */
    public function password()
    {
       if ($this->request->isPost()) {
            $input = input('post.');
            $save = UserModel::find($input['id']);
            $save->password = $input['password'];
            $save->save();
            return json(['status' => 'success', 'message' => '修改成功']);
       }
    }
    
    /**
     * 状态修改
     */
    public function status()
    {
       if ($this->request->isPost()) {
            $input = input('post.');
            $save = UserModel::find($input['id']);
            $save->status = $input['status'];
            $save->save();
            return json(['status' => 'success', 'message' => '修改成功']);
       }
    }
    
    /**
     * 批量修改
     */
    public function batch()
    {
        if ($this->request->isPost()) {
            $input = input('post.');
            foreach ($input['field'] as $key => $field) {
                UserModel::whereIn('id', $input['id'])->update([$field => $input[$field]]);
            }
            return json(['status' => 'success', 'message' => '修改成功']);
       }
    }
    
    /**
     * 删除指定资源
     */
    public function delete()
    {
        if ($this->request->isPost()) {
            UserModel::destroy(input('post.ids'));
            return json(['status' => 'success', 'message' => '删除成功']);
        }
    }
}
