<?php
// +----------------------------------------------------------------------
// | OneKeyAdmin [ Believe that you can do better ]
// +----------------------------------------------------------------------
// | Copyright (c) 2020-2023 http://onekeyadmin.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: MUKE <513038996@qq.com>
// +----------------------------------------------------------------------
namespace app\api\controller;

use think\exception\ValidateException;
use app\api\BaseController;
use app\api\addons\sendCode;
use app\api\model\UserToken;
use app\api\model\UserGroup;
use app\api\model\Config;
use app\api\model\User as UserModel;
use app\api\validate\User as UserValidate;
/**
 * 登录模块
 */
class Login extends BaseController
{
    /**
     * 账号密码登录
     */
    public function index()
    {
        if ($this->request->isPost()) {
            $locks = 5; // 最多尝试5次防止暴力破解
            $error = cache('user-login'.$this->request->ip());
            $error = empty($error) ? 0 : $error;
            if ($locks - $error == 0) {
                return json(['status' => 'error', 'message' => '您的账号被锁定，请1小时后再试或联系管理员！']);
            }
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('login')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            $userInfo = UserModel::where('mobile|email',$input['account'])->find();
            if (! $userInfo) {
                cache('user-login'.$this->request->ip(), $error + 1 ,3600);
                return json(['status' => 'error', 'message' => '密码或账号错误，您还有'.($locks - $error).'次机会']);
            }
            $password = UserModel::where('id', $userInfo->id)->value('password');
            if (! password_verify($input['password'], $password)) {
                cache('user-login'.$this->request->ip(), $error + 1 ,3600);
                return json(['status' => 'error', 'message' => '密码或账号错误，您还有'.($locks - $error).'次机会']);
            }
            if ($userInfo->status != 1) { 
                return json(['status' => 'error', 'message' => '您的账号被屏蔽']);
            }
            // 修改信息
            $userInfo->login_count = $userInfo->login_count + 1;
            $userInfo->login_ip    = $this->request->ip();
            $userInfo->login_time  = date('Y-m-d H:i:s');
            $userInfo->save();
            // 生成令牌
            $token = password_hash($userInfo->id . $this->request->ip() . $password, PASSWORD_BCRYPT, ['cost' => 12]);
            UserToken::create([
                'user_id'     => $userInfo->id,
                'token'       => $token,
                'create_time' => date('Y-m-d H:i:s')
            ]);
            $userInfo->group = UserGroup::where('integral', '<=', $userInfo->history_integral)->order('integral', 'desc')->find();
            $userInfo->token = $token;
            return json(['status' => 'success', 'message' => '登录成功', 'userInfo' => $userInfo]);
        }
    }
    
    /**
     * 手机快速登录注册
     */
    public function loginMobile()
    {
        if ($this->request->isPost()) {
            $locks = 5; // 最多尝试5次防止暴力破解
            $error = cache('user-login'.$this->request->ip());
            $error = empty($error) ? 0 : $error;
            if ($locks - $error == 0) {
                return json(['status' => 'error', 'message' => '您的账号被锁定，请1小时后再试或联系管理员！']);
            }
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('loginMobile')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            $userInfo = UserModel::where('mobile',$input['account'])->find();
            if ($input['code'] !== cache('login_mobile_code' . $input['account'])) {
                cache('user-login'.$this->request->ip(), $error + 1 ,3600);
                return json(['status' => 'error', 'message' => '验证码错误！']);
            }
            if ($userInfo) {
                if ($userInfo->status != 1) return json(['status' => 'error', 'message' => '您的账号被屏蔽']);
                // 修改信息
                $userInfo->login_count = $userInfo->login_count + 1;
                $userInfo->login_ip    = $this->request->ip();
                $userInfo->login_time  = date('Y-m-d H:i:s');
                $userInfo->save();
            } else {
                $config = Config::getVal('register');
                if (! $config) {
                    $config = ['nickname' => '未命名', 'describe' => '这个人很懒，什么都没有留下。', 'cover' => ''];
                }
                $date = date('Y-m-d H:i:s');
                $userInfo = UserModel::create([
                    'nickname'         => $config['nickname'],
                    'cover'            => $config['cover'],
                    'sex'              => 0,
                    'email'            => "",
                    'mobile'           => $input['account'],
                    'account'          => "",
                    'password'         => rand_id(16),
                    'pay_paasword'     => "",
                    'describe'         => $config['describe'],
                    'birthday'         => date('Y-m-d'),
                    'now_integral'     => 0,
                    'history_integral' => 0,
                    'balance'          => 0,
                    'login_ip'         => $this->request->ip(),
                    'login_count'      => 1,
                    'login_time'       => $date,
                    'update_time'      => $date,
                    'create_time'      => $date,
                    'status'           => 1,
                    'hide'             => 1,
                ]);
                // 绑定事件
                event('RegisterEnd', $userInfo);
            }
            // 生成令牌
            $token = password_hash($userInfo->id . $this->request->ip() . $userInfo->password, PASSWORD_BCRYPT, ['cost' => 12]);
            UserToken::create([
                'user_id'     => $userInfo->id,
                'token'       => $token,
                'create_time' => date('Y-m-d H:i:s')
            ]);
            $userInfo->group = UserGroup::where('integral', '<=', $userInfo->history_integral)->order('integral', 'desc')->find();
            $userInfo->token = $token;
            return json(['status' => 'success', 'message' => '登录成功', 'userInfo' => $userInfo]);
        }
    }

    /**
     * 发送登录短信验证码
     */
    public function sendLoginMobileCode()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('codeMobile')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            $result = sendCode::sms($input['mobile'], 'login_mobile_code', '26BEKytK3bCe');
            return json($result);
        }
    }
    
    /**
     * 邮箱找回密码
     */
    public function passwordEmail()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('passwordEmail')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            if ($input['code'] !== cache('password_email_code' . $input['email'])) {
                return json(["status" => "error", "message" => '邮箱验证码不正确']);
            }
            $save = UserModel::where('email', $input['email'])->find();
            if (! $save) {
                return json(['status' => 'error', 'message' => '账号不存在']);
            }
            $save->password = $input['password'];
            $save->save();
            return json(['status' => 'success', 'message' => '修改成功']);
        }
    }

    /**
     * 短信找回密码
     */
    public function passwordMobile()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('passwordMobile')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            if ($input['code'] !== cache('password_mobile_code' . $input['mobile'])) {
                return json(["status" => "error", "message" => '短信验证码不正确']);
            }
            $save = UserModel::where('mobile', $input['mobile'])->find();
            if (! $save) {
                return json(['status' => 'error', 'message' => '账号不存在']);
            }
            $save->password = $input['password'];
            $save->save();
            return json(['status' => 'success', 'message' => '修改成功']);
        }
    }
    
    /**
     * 邮箱注册
     */
    public function registerEmail()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('registerEmail')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            if ($input['code'] !== cache('register_email_code' . $input['email'])) {
                return json(["status" => "error", "message" => '邮箱验证码不正确']);
            }
            if (UserModel::where('email', $input['email'])->value('id')) {
                return json(['status' => 'error', 'message' => '邮箱号已被注册']);
            }
            $config = Config::getVal('register');
            if (! $config) {
                $config = ['nickname' => '未命名', 'describe' => '这个人很懒，什么都没有留下。', 'cover' => ''];
            }
            $date   = date('Y-m-d H:i:s');
            $registerInfo = UserModel::create([
                'nickname'         => $config['nickname'],
                'cover'            => $config['cover'],
                'sex'              => 0,
                'email'            => $input['email'],
                'mobile'           => "",
                'account'          => "",
                'password'         => $input['password'],
                'pay_paasword'     => "",
                'describe'         => $config['describe'],
                'birthday'         => date('Y-m-d'),
                'now_integral'     => 0,
                'history_integral' => 0,
                'balance'          => 0,
                'login_ip'         => "",
                'login_count'      => 0,
                'login_time'       => $date,
                'update_time'      => $date,
                'create_time'      => $date,
                'status'           => 1,
                'hide'             => 1,
            ]);
            // 绑定事件
            event('RegisterEnd', $registerInfo);
            return json(['status' => 'success', 'message' => '注册成功']);
        }
    }
    
    /**
     * 手机注册
     */
    public function registerMobile()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('registerMobile')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            if ($input['code'] !== cache('register_mobile_code' . $input['mobile'])) {
                return json(["status" => "error", "message" => '短信验证码不正确']);
            }
            if (UserModel::where('mobile', $input['mobile'])->value('id')) {
                return json(['status' => 'error', 'message' => '手机号已被注册']);
            }
            $config = Config::getVal('register');
            if (! $config) {
                $config = ['nickname' => '未命名', 'describe' => '这个人很懒，什么都没有留下。', 'cover' => ''];
            }
            $date = date('Y-m-d H:i:s');
            $registerInfo = UserModel::create([
                'nickname'         => $config['nickname'],
                'cover'            => $config['cover'],
                'sex'              => 0,
                'email'            => "",
                'mobile'           => $input['mobile'],
                'account'          => "",
                'password'         => $input['password'],
                'pay_paasword'     => "",
                'describe'         => $config['describe'],
                'birthday'         => date('Y-m-d'),
                'now_integral'     => 0,
                'history_integral' => 0,
                'balance'          => 0,
                'login_ip'         => '',
                'login_count'      => 0,
                'login_time'       => $date,
                'update_time'      => $date,
                'create_time'      => $date,
                'status'           => 1,
                'hide'             => 1,
            ]);
            // 绑定事件
            event('RegisterEnd', $registerInfo);
            return json(['status' => 'success', 'message' => '注册成功']);
        }
    }

    /**
     * 发送注册邮箱验证码
     */
    public function sendRegisterEmailCode()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('codeEmail')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            if (UserModel::where('email', $input['email'])->value('id')) {
                return json(['status' => 'error', 'message' => '邮箱号已被注册']);
            }
            $result = sendCode::email($input['email'], 'register_email_code','用户注册');
            return json($result);
        }
    }

    /**
     * 发送注册短信验证码
     */
    public function sendRegisterMobileCode()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('codeMobile')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            if (UserModel::where('mobile', $input['mobile'])->value('id')) {
                return json(['status' => 'error', 'message' => '手机号已被注册']);
            }
            $result = sendCode::sms($input['mobile'], 'register_mobile_code', '26BEKytK3bCe');
            return json($result);
        }
    }

    /**
     * 发送修改密码邮箱验证码
     */
    public function sendPasswordEmailCode()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('codeEmail')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            if (UserModel::where('email', $input['email'])->value('id')) {
                $result = sendCode::email($input['email'], 'password_email_code', '修改密码');
                return json($result);
            } else {
                return json(['status' => 'error', 'message' => '邮箱号未注册']);
            }
        }
    }

    /**
     * 发送修改密码短信验证码
     */
    public function sendPasswordMobileCode()
    {
        if ($this->request->isPost()) {
            try {
                $input = input('post.');
                validate(UserValidate::class)->scene('codeMobile')->check($input);
            } catch ( ValidateException $e ) {
                return json(['status' => 'error', 'message' => $e->getError()]);
            }
            if (UserModel::where('mobile', $input['mobile'])->value('id')) {
                $result = sendCode::sms($input['mobile'], 'password_mobile_code', '26BEKytK3bCe');
                return json($result);
            } else {
                return json(['status' => 'error', 'message' => '手机号未注册']);
            }
        }
    }
}