/**
 * 统一表格
 */
Vue.component('el-curd', {
    template: `
    <div class="el-curd">
        <el-form 
            ref="search" 
            class="el-curd-header" 
            size="small" 
            :inline="true" 
            :model="search" 
            @submit.native.prevent>
            <el-form-item class="el-button-form">
                <el-button 
                    v-if="curdDeleteAuthority && tableSelection" 
                    :disabled="rows.length === 0"
                    type="danger" 
                    icon="el-icon-delete" 
                    @click="removeData()">
                    删除
                </el-button>
                <el-button 
                    v-if="curdSaveAuthority" 
                    type="primary" 
                    icon="el-icon-plus" 
                    @click="openData()">
                    {{saveTitle}}
                </el-button>
                <el-dropdown v-if="batch.length > 0" @command="batchData">
                    <el-button type="info">
                        批量修改<i class="el-icon-arrow-down el-icon--right"></i>
                    </el-button>
                    <el-dropdown-menu slot="dropdown">
                        <el-dropdown-item v-for="(item, index) in batch" :command="item.field">{{item.title}}</el-dropdown-item>
                    </el-dropdown-menu>
                </el-dropdown>
                <el-dropdown v-if="tableExport" @command="exportData">
                    <el-button icon="el-icon-download">导出</el-button>
                    <el-dropdown-menu slot="dropdown">
                        <el-dropdown-item command="csv">CSV</el-dropdown-item>
                        <el-dropdown-item command="json">JSON</el-dropdown-item>
                        <el-dropdown-item command="xhtml">XHTML</el-dropdown-item>
                        <el-dropdown-item command="txt">TXT</el-dropdown-item>
                    </el-dropdown-menu>
                </el-dropdown>
                <slot name="button"></slot>
            </el-form-item>
            <el-form-item class="el-search-form">
                <el-date-picker 
                    v-if="searchDate"
                    v-model="search.date" 
                    type="daterange" 
                    align="right" 
                    unlink-panels 
                    range-separator="至" 
                    start-placeholder="开始日期" 
                    end-placeholder="结束日期" 
                    format="yyyy-MM-dd" 
                    value-format="yyyy-MM-dd" 
                    :picker-options="picker"
                    @change="searchData()">
                </el-date-picker>
                <el-input 
                    v-if="searchKeyword"
                    :placeholder="searchKeyword === true ? '根据关键词搜索' : searchKeyword" 
                    v-model="search.keyword"
                    @keyup.enter.native="searchData()">
                    <el-button slot="append" icon="el-icon-search" @click="searchData()"></el-button>
                </el-input>
                <el-button 
                    v-if="searchCatalog.length > 0 || searchStatus.length > 0 || $slots.search" 
                    size="small" 
                    icon="el-icon-arrow-up"
                    title="展开搜索"
                    @click="searchDrawer = true">
                </el-button>
                <el-drawer
                    direction="ttb"
                    :visible.sync="searchDrawer"
                    :with-header="false"
                    size="70px">
                    <div class="el-search-form-warp">
                        <div class="el-search-item" v-if="searchCatalog.length > 0">
                            <div class="label">分类搜索：</div>
                            <div class="value">
                                <el-select class="el-search-item" v-model="search.catalog" placeholder="查看所有分类" filterable @change="searchData()">
                                    <el-option label="全部分类" value=""></el-option>
                                    <el-option v-for="(item, index) in searchCatalog" :key="index" :label="item.title" :value="item.id">
                                        <span v-html="item.treeString"></span>{{ item.title }}
                                    </el-option>
                                </el-select>
                            </div>
                        </div>
                        <div class="el-search-item" v-if="searchStatus.length > 0">
                            <div class="label">状态搜索：</div>
                            <div class="value">
                                <el-select class="el-search-item" v-model="search.status" placeholder="查看所有状态" @change="searchData()">
                                    <el-option label="全部状态" value=""></el-option>
                                    <el-option v-for="(item, index) in searchStatus" :key="index" :label="item.label" :value="item.value"></el-option>
                                </el-select>
                            </div>
                        </div>
                        <slot name="search"></slot>
                    </div>
                </el-drawer>
            </el-form-item>
        </el-form>
        <div class="el-curd-table" :class="{'el-curd-table-tree': tableTree}">
            <el-table 
                ref="table"
                v-loading="loading"
                :row-key="rowKey"
                :data="table"
                :tree-props="{children: 'children', hasChildren: 'hasChildren'}"
                :highlight-current-row="true"
                :row-class-name="tableRowClass"
                :lazy="tableTree"
                :load="tableLoad"
                @select="selectRow"
                @select-all="selectAll"
                @selection-change="selectionChange"
                @sort-change="sortChange">
                <el-table-column v-if="tableSelection" type="selection" width="55" :selectable="selectableDisabled"></el-table-column>
                <template v-for="(item, index) in column">
                    <el-table-column
                        :key="index"
                        :prop="item.sortProp" 
                        :width="item.table.width" 
                        :label="typeof item.table.expand == 'undefined' || item.table.expand == false ? typeof item.table.label !== 'undefined' ? item.table.label : item.label : ''" 
                        :type="typeof item.table.expand !== 'undefined' && item.table.expand ? 'expand' : undefined"
                        :sortable="typeof item.table.sort !== 'undefined' && item.table.sort ? 'custom' : false">
                        <template slot-scope="scope">
                            <el-image 
                                style="height: 40px;width: 40px"
                                v-if="item.table.is === 'el-image'"
                                :src="scope.row[item.prop]" 
                                :preview-src-list="[scope.row[item.prop]]">
                                <div slot="error" class="image-slot">
                                    <img class="error-image" src="/admin/images/error.png"/>
                                </div>
                            </el-image>
                            <template v-if="item.table.is === 'el-images' && typeof scope.row[item.prop] == 'object'">
                                <el-image 
                                    style="margin-right: 5px"
                                    v-for="(child, index) in scope.row[item.prop]"
                                    :index="index"
                                    :src="child.url" 
                                    :preview-src-list="arrayColumn(scope.row[item.prop], 'url')"
                                    :style="{
                                        height: typeof item.table.imageWidth == 'undefined' ? '40px' : item.table.imageWidth, 
                                        width: typeof item.table.imageWidth == 'undefined' ? '40px' : item.table.imageWidth
                                    }">
                                    <div slot="error" class="image-slot">
                                        <img class="error-image" src="/admin/images/error.png"/>
                                    </div>
                                </el-image>
                            </template>
                            <template v-if="item.table.is === 'object' && scope.row[item.prop] != null">
                                <div v-for="(child, index) in item.table.child" :key="index" 
                                    v-html="scope.row[item.prop][child]">
                                </div>
                            </template>
                            <template v-if="item.table.is === 'el-html'">
                                <div v-html="scope.row[item.prop]"></div>
                            </template>
                            <span v-if="item.table.is === 'el-text'">
                                {{scope.row[item.prop]}}
                            </span>
                            <el-switch
                                v-if="item.table.is === 'el-switch'"
                                v-model="scope.row[item.prop]"
                                :active-value="1"
                                :inactive-value="0"
                                :active-text="typeof item.form.activeText == 'undefined' ? '正常' : item.form.activeText"
                                :inactive-text="typeof item.form.inactiveText == 'undefined' ? '屏蔽' : item.form.inactiveText"
                                :disabled="scope.row.disabled"
                                @change="oneKeySaveData(scope.row)">
                            </el-switch>
                            <el-input 
                                v-if="item.table.is === 'el-input'"
                                class="el-curd-table-input" 
                                v-model="scope.row[item.prop]" 
                                size="small" 
                                :disabled="scope.row.disabled" 
                                @change="oneKeySaveData(scope.row)">
                            </el-input>
                            <span v-if="typeof item.table.is === 'undefined'">
                                <template 
                                    v-if="(item.form.is == 'el-radio-group' || 
                                    item.form.is == 'el-checkbox-group' || 
                                    item.form.is == 'el-select' || 
                                    item.form.is == 'el-switch') &&
                                    item.prop != item.table.prop">
                                    <template v-if="item.form.is == 'el-switch'">
                                        <el-tag v-if="scope.row[item.prop] == 0" type="info" effect="dark" size="small">{{typeof item.form.inactiveText != 'undefined' ? item.form.inactiveText : '屏蔽'}}</el-tag>
                                        <el-tag v-else effect="dark" size="small">{{typeof item.form.activeText != 'undefined' ? item.form.activeText : '正常'}}</el-tag>
                                    </template>
                                    <template v-else>
                                        <template v-for="(val, key) in formChildValue(item)">
                                            <template v-if="typeof val.value == 'undefined'">
                                                <template v-if="typeof scope.row[item.prop] == 'object'">
                                                    <el-tag v-if="scope.row[item.prop].indexOf(val.label) != -1" size="small" style="margin-right:5px">{{val.label}}</el-tag>
                                                </template>
                                                <template v-else>
                                                    <el-tag v-if="scope.row[item.prop] == val.label" size="small" style="margin-right:5px">{{val.label}}</el-tag>
                                                </template>
                                            </template>
                                            <template v-else>
                                                <template v-if="typeof scope.row[item.prop] == 'object'">
                                                    <el-tag v-if="scope.row[item.prop].indexOf(val.value) != -1" size="small" style="margin-right:5px">{{val.label}}</el-tag>
                                                </template>
                                                <template v-else>
                                                    <el-tag v-if="scope.row[item.prop] == val.value" size="small" style="margin-right:5px">{{val.label}}</el-tag>
                                                </template>
                                            </template>
                                        </template>
                                    </template>
                                </template>
                                <span v-else v-html="scope.row[item.prop]"></span>
                            </span>
                            <template v-if="typeof item.table.bind !== 'undefined'">
                                <div v-for="(bind, index) in item.table.bind" :key="index" v-html="scope.row[bind]"></div>
                            </template>
                        </template>
                    </el-table-column>
                </template>
                <el-table-column v-if="operationWidth > 0 || $scopedSlots.operation || $scopedSlots.dropdown" label="操作" :width="operationWidth" align="center">
                    <template slot-scope="scope">
                        <div class="operation" v-if="scope.row.disabled !== true">
                            <a 
                                href="javascript:;" 
                                v-if="curdDeleteAuthority && scope.row.delete_disabled !== true" 
                                @click="removeData(scope.row)">
                                删除
                            </a>
                            <a 
                                href="javascript:;" 
                                v-if="curdUpdateAuthority && scope.row.update_disabled !== true" 
                                @click="openData(scope.row)">
                                {{updateTitle}}
                            </a>
                            <slot name="operation" v-bind="scope.row"></slot>
                            <el-dropdown v-if="copyAuthority || curdPushAuthority || previewAuthority || $scopedSlots.dropdown">
                                <a href="javascript:;" class="el-dropdown-link">
                                    更多<i class="el-icon-arrow-down el-icon--right"></i>
                                </a>
                                <el-dropdown-menu slot="dropdown">
                                    <li v-if="curdSaveAuthority && copyAuthority" @click="copyData(scope.row)">复制</li>
                                    <li v-if="curdPushAuthority" @click="pushData(scope.row)">追加</li>
                                    <li v-if="previewAuthority" @click="previewData(scope.row)">预览</li>
                                    <slot name="dropdown" v-bind="scope.row"></slot>
                                </el-dropdown-menu>
                            </el-dropdown>
                        </div>
                    </template>
                </el-table-column>
            </el-table>
        </div>
        <el-pagination
            v-if="! tableTree"
            layout="total, sizes, prev, pager, next, jumper"
            :current-page="search.page"
            :page-size="search.pageSize"
            :page-sizes="pageSizes"
            :total="total"
            background
            @size-change="pageSizeChange"
            @current-change="pageChange">
        </el-pagination>
        <transition>
            <keep-alive>
                <component
                    top="20px" 
                    size="100%" 
                    class="drawer" 
                    width="500px"
                    :is="drawerAppoint == '' ? 'el-drawer' : 'el-dialog'"
                    :visible.sync="drawer" 
                    :with-header="false" 
                    :title="drawerData[rowKey] === '' ? saveTitle : drawerTitle === '' ? updateTitle : drawerTitle"
                    :close-on-click-modal="false">
                    <el-page-header @back="drawer=false" :content="drawerData[rowKey] === '' ? saveTitle : updateTitle">
                        <template v-slot:title>Esc键返回</template>
                    </el-page-header>
                    <el-form 
                        ref="drawerData" 
                        class="el-layout" 
                        :class="{'el-layout-one': colRowShow.length === 1, 'el-layout-appoint': drawerAppoint != ''}" 
                        :rules="drawerRules" 
                        :model="drawerData" 
                        :label-width="formLabelWidth" 
                        :validate-on-rule-change="false"
                        @submit.native.prevent>
                        <el-tabs :tab-position="position">
                            <el-tab-pane v-for="(tab, key) in colRowShow" :name="String(key)" :index="key">
                                <span slot="label">
                                    <i :class="tab.icon" v-if="tab.icon"></i>{{tab.label}}
                                </span>
                                <div :class="tab.warp === false ? '' : 'el-pane-warp'">
                                    <el-row :gutter="formGutter">
                                        <div v-for="(item, index) in tab.field" :key="index">
                                            <el-col 
                                                v-if="formItemShow(item)" 
                                                :md="typeof item.form.colMd == 'undefined' ? formColMd : item.form.colMd" 
                                                :xs="typeof item.form.colSm == 'undefined' ? formColSm : item.form.colSm">
                                                <el-form-item :prop="item.prop" :label-width="typeof item.form.labelWidth != 'undefined' ? item.form.labelWidth : ''">
                                                    <template v-slot:label>
                                                        <el-tooltip placement="top" :content="formVariable(item)" :disabled="variable == ''">
                                                            <i>{{item.label == '' ? '' : item.label + '：'}}</i>
                                                        </el-tooltip>
                                                    </template>
                                                    <component 
                                                        class="el-component"
                                                        v-model="drawerData[item.prop]"
                                                        :is="item.form.is"
                                                        :key="drawerData[rowKey] + index + randId"
                                                        :type="item.form.type"
                                                        :style="item.form.style"
                                                        :href="item.form.href"
                                                        :limit="item.form.limit"
                                                        :height="item.form.height"
                                                        :editorcss="item.form.editorcss"
                                                        :options="item.form.options"
                                                        :disabled="item.form.disabled"
                                                        :placeholder="item.form.placeholder"
                                                        :filterable="item.form.filterable"
                                                        :multiple="item.form.multiple"
                                                        :remote="typeof item.form.remote !== 'undefined'"
                                                        :remote-method="typeof item.form.remote !== 'undefined' ? (query)=>{remoteMethod(query, item.form)} : undefined"
                                                        :props="item.form.props"
                                                        :label-position="item.form.labelPosition"
                                                        :label-width="typeof item.form.labelWidth != 'undefined' &&  item.form.labelWidth != '0px' ? item.form.labelWidth : formLabelWidth"
                                                        :data="typeof item.form.data  == 'undefined' ? undefined : drawerData"
                                                        :format="item.form.format"
                                                        :value-format="item.form.valueFormat"
                                                        :is-range="item.form.type == 'is-range'"
                                                        :maxlength="typeof item.form.maxlength != 'undefined' ? item.form.maxlength : undefined"
                                                        :minlength="typeof item.form.minlength != 'undefined' ? item.form.minlength : undefined"
                                                        :max="typeof item.form.max != 'undefined' ? item.form.max : undefined"
                                                        :min="typeof item.form.min != 'undefined' ? item.form.min : undefined"
                                                        :controls-position="item.form.is  == 'el-input-number' ? 'right' : undefined"
                                                        :active-value="item.form.is == 'el-switch' ? 1 : undefined"
                                                        :inactive-value="item.form.is == 'el-switch' ? 0 : undefined"
                                                        :search="item.form.search"
                                                        :ifset="item.form.ifset"
                                                        :list="item.form.list"
                                                        :active-text="typeof item.form.activeText == 'undefined' ? '正常' : item.form.activeText"
                                                        :inactive-text="typeof item.form.inactiveText == 'undefined' ? '屏蔽' : item.form.inactiveText"
                                                        @input="formRules(item.form.is, item.prop)"
                                                        @drawer-data-change="changeData($event)">
                                                        <template v-if="typeof item.form.child !== 'undefined'">
                                                            <template v-if="item.form.is == 'el-radio-group' || item.form.is == 'el-checkbox-group'">
                                                                <component 
                                                                    v-for="(val, key) in formChildValue(item)"
                                                                    :is="item.form.is == 'el-radio-group' ? 'el-radio' : 'el-checkbox'"
                                                                    :key="val.label"
                                                                    :label="typeof val.value == 'undefined' ? val.label : val.value">
                                                                    {{ val.label }}
                                                                </component>
                                                            </template>
                                                            <template v-else>
                                                                <component 
                                                                    v-for="(val, key) in formChildValue(item)"
                                                                    is="el-option"
                                                                    :key="val.value"
                                                                    :label="val.label"
                                                                    :value="typeof val.value == 'undefined' ? val.label : val.value">
                                                                    <span v-html="val.treeString"></span>
                                                                    {{ val.label }}
                                                                </component>
                                                            </template>
                                                        </template>
                                                    </component>
                                                    <div class="el-tips" v-if="typeof item.form.tips !== 'undefined'" v-html="item.form.tips"></div>
                                                </el-form-item>
                                            </el-col>
                                        </div>
                                    </el-row>
                                    <div v-if="tab.warp !== false && drawerSave !== false" class="el-bottom">
                                        <el-button 
                                            size="medium" 
                                            type="primary" 
                                            icon="el-icon-refresh-right" 
                                            :loading="drawerLoading"
                                            @click="saveData()" >
                                            保 存
                                        </el-button>
                                        <el-button size="medium" @click="drawer = false">关 闭</el-button>
                                    </div>
                                </div>
                            </el-tab-pane>
                        </el-tabs>
                    </el-form>
                </component>
            </keep-alive>
        </transition>
    </div>
    `,
    props: {
        field: {
            type: Array,
            required: true
        },
        batch: {
            type: Array,
            default: () => {
                return [];
            },
        },
        indexUrl: {
            type: String,
            default: controller + '/index',
        },
        saveUrl: {
            type: String,
            default: controller + '/save',
        },
        updateUrl: {
            type: String,
            default: controller + '/update',
        },
        deleteUrl: {
            type: String,
            default: controller + '/delete',
        },
        batchUrl: {
            type: String,
            default: controller + '/batch',
        },
        saveTitle: {
            type: String,
            default: '新增',
        },
        updateTitle: {
            type: String,
            default: '编辑',
        },
        deleteTitle: {
            type: String,
            default: '删除',
        },
        copyAuthority: {
            type: Boolean,
            default: false,
        },
        saveAuthority: {
            type: Boolean,
            default: true,
        },
        deleteAuthority: {
            type: Boolean,
            default: true,
        },
        updateAuthority: {
            type: Boolean,
            default: true,
        },
        pushAuthority: {
            type: Boolean,
            default: true,
        },
        previewAuthority: {
            type: Boolean,
            default: false,
        },
        tableTree: {
            type: Boolean,
            default: false
        },
        tableTreeProps: {
            default: () => {
                return {parent: 'pid', label: 'title'};
            }
        },
        tableExport:{
            type: Boolean,
            default: true,
        },
        tableSort: {
            type: Object,
            default: () => {
                return {};
            },
        },
        tableOperationWidth: {
            type: String,
            default: '',
        },
        tablePageSize: {
            type: Number,
            default: 20,
        },
        tablePageSizes: {
            type: Array,
            default: () => {
                return [20, 50, 100, 200, 500];
            },
        },
        tableEmpty: {
            type: String,
            default: '暂无数据',
        },
        tableSelection: {
            type: Boolean,
            default: true
        },
        searchCatalog: {
            type: Array,
            default: () => {
                return [];
            },
        },
        searchStatus: {
            type: Array,
            default: () => {
                return [];
            },
        },
        searchKeyword: {
            default: true
        },
        searchInit: {
            type: Boolean,
            default: true
        },
        searchDate: {
            type: Boolean,
            default: true
        },
        formLabelWidth: {
            type: String,
            default: '120px'
        },
        formGutter: {
            type: Number,
            default: 10,
        },
        formColMd: {
            type: Number,
            default: 24,
        },
        formColSm: {
            type: Number,
            default: 24,
        },
        variable: {
            type: String,
            default: ''
        },
        rowKey: {
            type: String,
            default: 'id'
        },
    },
    data() {
        return {
            position: document.body.clientWidth > 768 ? 'left' : 'top',
            randId: common.id(16),
            catalogUrl: "api/catalog",
            column: [],
            colRow: [],
            colRowShow: [],
            pNode: '',
            sNode: new Map(),
            rows: [],
            list: [],
            table: [],
            searchDrawer: false,
            activeRows: [],
            search: {
                keyword: '',
                date: '',
                catalog: '',
                status: '',
                prop: '',
                order: '',
                page: 1,
                pageSize: this.tablePageSize,
                theme: theme,
                param: locationUrl.get('param'),
            },
            pageSizes: this.tablePageSizes,
            total: 0,
            loading: false,
            select: false,
            drawer: false,
            drawerData: {},
            drawerForm: {},
            drawerRules: {},
            drawerAppoint: "",
            drawerLoading: false,
            drawerTitle: '',
            drawerSave: true,
            drawerUrl: "",
            picker: {
                shortcuts: [{
                    text: '今天',
                    onClick(picker) {
                        const end = new Date();
                        const start = new Date();
                        end.setTime(start.getTime() + 3600 * 1000 * 24 * 1);
                        start.setTime(start.getTime() + 3600 * 1000);
                        picker.$emit('pick', [start, end]);
                    }
                }, {
                    text: '昨天',
                    onClick(picker) {
                        const end = new Date();
                        const start = new Date();
                        end.setTime(start.getTime() + 3600 * 1000);
                        start.setTime(start.getTime() - 3600 * 1000 * 24 * 1);
                        picker.$emit('pick', [start, end]);
                    }
                }, {
                    text: '最近7天',
                    onClick(picker) {
                        const end = new Date();
                        const start = new Date();
                        end.setTime(start.getTime() + 3600 * 1000 * 24 * 1);
                        start.setTime(start.getTime() - 3600 * 1000 * 24 * 7);
                        picker.$emit('pick', [start, end]);
                    }
                }, {
                    text: '最近30天',
                    onClick(picker) {
                        const end = new Date();
                        const start = new Date();
                        end.setTime(start.getTime() + 3600 * 1000 * 24 * 1);
                        start.setTime(start.getTime() - 3600 * 1000 * 24 * 30);
                        picker.$emit('pick', [start, end]);
                    }
                }, {
                    text: '本月',
                    onClick(picker) {
                        let year = new Date().getFullYear();
                        let month = new Date().getMonth();
                        const end = new Date();
                        const start = new Date(year, month, 1);
                        end.setTime(end.getTime() + 3600 * 1000 * 24 * 1);
                        start.setTime(start.getTime());
                        picker.$emit('pick', [start, end]);
                    }
                }, {
                    text: '本年',
                    onClick(picker) {
                        let year = new Date().getFullYear();
                        const end = new Date();
                        const start = new Date(year, 0);
                        end.setTime(end.getTime() + 3600 * 1000 * 24 * 1);
                        start.setTime(start.getTime());
                        picker.$emit('pick', [start, end]);
                    }
                }]
            },
        }
    },
    mounted() {
        let self = this;
        // 标签页
        window.onresize = function(){
            self.position = document.body.clientWidth > 768 ? 'left' : 'top';
        };
    },
    computed: {
        curdSaveAuthority() {
            return authority.indexOf(this.saveUrl) !== -1 && this.saveAuthority;
        },
        curdUpdateAuthority() {
            return authority.indexOf(this.updateUrl) !== -1 && this.updateAuthority;
        },
        curdDeleteAuthority() {
            return authority.indexOf(this.deleteUrl) !== -1 && this.deleteAuthority;
        },
        curdPushAuthority() {
            return this.curdSaveAuthority && this.tableTree && this.pushAuthority;
        },
        operationWidth() {
            if (this.tableOperationWidth === '') {
                let width = 0;
                if (this.curdUpdateAuthority) width += 60;
                if (this.curdDeleteAuthority) width += 60;
                if (this.copyAuthority || this.curdPushAuthority || this.previewAuthority || this.$scopedSlots.dropdown) {
                    width += 65;
                }
                return width;
            } else {
                return this.tableOperationWidth;
            }
        },
    },
    created() {
        let self = this;
        self.search.prop  = typeof self.tableSort.prop === 'undefined' ? '' : self.tableSort.prop,
        self.search.order = typeof self.tableSort.order === 'undefined' ? '' : self.tableSort.order
        self.colRow = typeof self.field[0]['field'] === 'undefined' ? [{label: '基础信息', field: self.field}] : self.field;
        self.colRow.forEach(function (items, index) {
            items.field.forEach(function (item, index) {
                // 表单
                item.form = typeof item.form === 'undefined' ? {} : item.form;
                item.form.is = typeof item.form.is === 'undefined' ? 'el-input' : item.form.is;
                self.drawerForm[item.prop] = typeof item.form.default === 'undefined' ? '' : item.form.default;
                // 表格
                item.table = typeof item.table === 'undefined' ? {} : item.table;
                if (item.table) {
                    let itemTable = JSON.parse(JSON.stringify(item));
                    itemTable.sortProp = item.prop;
                    itemTable.prop = typeof item.table.prop !== 'undefined' ? item.table.prop : item.prop;
                    itemTable.label = typeof item.table.label !== 'undefined' && item.table.label !== '' ? item.table.label : item.label;
                    self.column.push(itemTable);
                }
            })
        });
        self.colRowShow = JSON.parse(JSON.stringify(self.colRow));
        if (self.searchInit) {
            self.getData();
        }
        self.formRules();
    },
    methods: {
        /**
         * 获取数据
         */
        getData() {
            let self = this;
            self.loading = true;
            if (self.search.order == null) {
                if (JSON.stringify(self.tableSort) === "{}") {
                    self.search.order = '';
                    self.search.prop  = '';
                } else {
                    self.search.order = self.tableSort.order;
                    self.search.prop  = self.tableSort.prop;
                }
            } else {
                self.search.order = self.search.order === 'ascending' || self.search.order === 'asc' ? 'asc' : 'desc';
            }
            request.post(self.indexUrl, self.search, function(res) {
                if (res.status === 'success') {
                    self.total = res.count;
                    self.list  = res.data;
                    self.table = self.tableTree && self.search.keyword === '' ? tree.convert(self.list, self.tableTreeProps.parent, self.rowKey) : self.list;
                    // 解决树形表格数据量多时卡顿问题
                    if (self.tableTree && self.search.keyword === '') {
                        self.table.forEach(function (item, index) {
                            item.hasChildren = item.children && item.children.length > 0;
                            item.children = null;
                            return item
                        })
                        self.getChilderData();
                    }
                    self.$emit('get-data', res);
                } else {
                    self.$notify({ showClose: true, message: res.message, type: res.status});
                }
                self.loading = false;
            });
        },
        /**
         * 将树数据转化为平铺数据
         */
        treeToTile(treeData, childKey = 'children') {
            const arr = []
            const expanded = data => {
                if (data && data.length > 0) {
                    data.filter(d => d).forEach(e => {
                        arr.push(e)
                        expanded(e[childKey] || [])
                    })
                }
            }
            expanded(treeData)
            return arr;
        },
        /**
         * 树形拖动数据时，需要展开的父级
         */
        parentList(parentId, arr = []) {
            let self = this;
            if (parentId != 0) {
                self.list.forEach(function (item, index) {
                    if (parentId == item[self.rowKey]) {
                        arr.push(item);
                        self.parentList(item[self.tableTreeProps.parent], arr);
                    }
                })
            }
            return arr;
        },
        /**
         * 树形修改数据（树形表格特殊处理防止页面卡顿）
         */
        getChilderData() {
            let self   = this;
            if (self.tableTree && self.search.keyword === '') {
                let parent = self.drawerData[self.tableTreeProps.parent];
                // 刷新当前父级子节点
                if (parent != 'undefined' && parent != 0) {
                    let children = [];
                    self.list.forEach(function (item, index) {
                        if (item[self.tableTreeProps.parent] === parent) {
                            children.push(item);
                            if (typeof item.children != 'undefined' && item.children != null && item.children.length > 0) {
                                self.$set(self.$refs.table.store.states.lazyTreeNodeMap, item[self.rowKey], item.children);
                            }
                        }
                    });
                    self.$set(self.$refs.table.store.states.lazyTreeNodeMap, parent, children);
                }
                // 刷新之前父级子节点
                if (self.pNode !== '' && self.pNode !== parent) {
                    let pNodes = Array.isArray(self.pNode) ? self.pNode : [self.pNode];
                    pNodes.forEach(function (pNode, index) {
                        let children = [];
                        self.list.forEach(function (item, index) {
                            if (item[self.tableTreeProps.parent] === pNode) {
                                children.push(item);
                                if (typeof item.children != 'undefined' && item.children != null && item.children.length > 0) {
                                    self.$set(self.$refs.table.store.states.lazyTreeNodeMap, item[self.rowKey], item.children);
                                }
                            }
                        });
                        if (typeof self.sNode.get(pNode) != 'undefined') {
                            self.$set(self.$refs.table.store.states.lazyTreeNodeMap, pNode, children);
                        }
                    })
                }
            }
        },
        /**
         * 查看数据
         */
        openData(row = "", drawerAppoint = "", drawerTitle = "", drawerSave = true) {
            this.drawerAppoint    = drawerAppoint;
            this.drawerSave       = drawerSave;
            this.drawerTitle      = drawerTitle;
            this.drawerUrl        = typeof drawerSave === 'string' ? drawerSave : '';
            this.drawerData       = row === "" ? JSON.parse(JSON.stringify(this.drawerForm)) : JSON.parse(JSON.stringify(row));
            this.drawerData.theme = theme;
            this.drawer = true;
            if (this.tableTree && this.search.keyword === '') {
                this.pNode = row === "" ? "" : JSON.parse(JSON.stringify(row[this.tableTreeProps.parent]));
            }
            this.$emit('open-data', this.drawerData);
        },
        /**
         * 批量修改数据
         */
        batchData(field) {
            if (this.rows.length == 0) {
                this.$notify({ showClose: true, message: '请选中要设置的行', type: 'warning'});
                return false;
            }
            this.drawerAppoint = field;
            this.drawerSave    = true;
            this.drawerTitle   = '批量修改';
            this.drawerUrl     = this.batchUrl;
            this.drawerData    = JSON.parse(JSON.stringify(this.drawerForm));
            this.drawerData[this.rowKey] = common.arrayColumn(this.rows, this.rowKey);
            this.drawerData[theme]       = theme;
            this.drawerData['field']     = field;
            this.drawer = true;
            this.$emit('open-data', this.drawerData);
        },
        /**
         * 改变数据
         */
        changeData(data) {
            this.drawerData = data;
        },
        /**
         * 快捷修改数据
         */
        oneKeySaveData(row) {
            let self = this;
            self.drawerData = row;
            request.post(self.updateUrl, row, function(res){
                if (res.status === 'success') {
                    self.$emit('save-data', res);
                    self.getData();
                } else {
                    self.$notify({ showClose: true, message: res.message, type: res.status});
                }
            });
        },
        /**
         * 保存数据
         */
        saveData() {
            let self = this;
            self.$refs.drawerData.validate((valid) => {
                if (valid) {
                    self.drawerLoading = true;
                    let url = self.drawerData[self.rowKey] === '' ? self.saveUrl : self.drawerUrl === '' ? self.updateUrl : self.drawerUrl;
                    request.post(url, self.drawerData, function(res){
                        self.drawerLoading = false;
                        if (res.status === 'success') {
                            self.getData();
                            self.drawer = false;
                            self.$emit('save-data', res);
                        }
                        self.$notify({ showClose: true, message: res.message, type: res.status});
                    });
                } else {
                    return false;
                }
            });
        },
        /**
         * 导出数据
         */
        exportData(type) {
            let self = this;
            let list = self.tableTree ? tree.convertString(self.list, self.tableTreeProps.parent, self.rowKey) : self.list;
            let str  = '';
            if (type === 'json') {
                str = JSON.stringify(list);
            } else {
                let props = [];
                let labels = [];
                self.column.forEach(function (item, index) {
                    if (typeof item.table.expand === 'undefined' || item.table.expand === false) {
                        let prop  = item.prop;
                        let label = item.label;
                        if (typeof item.table.bind !== 'undefined' && item.table.bind.length > 0) {
                            prop = [prop];
                            item.table.bind.forEach(function (b,i) {
                                prop.push(b);
                            })
                        }
                        if (typeof item.table.child !== 'undefined' && item.table.child.length > 0) {
                            let key = prop;
                            prop = [];
                            item.table.child.forEach(function (b,i) {
                                prop.push(key + '.' + b);
                            })
                        }
                        props.push(prop);
                        labels.push(label);
                    }
                });
                str = labels.toString() + '\n';
                let rows = [];
                list.forEach(function (item, index) {
                    props.forEach(function (prop, index) {
                        if ((typeof prop == 'object')) {
                            prop.forEach(function (b,i) {
                                let g = prop.length - 1 == i ? '' : '|';
                                let k = b.split('.');
                                let v = item[prop[i]];
                                let s = typeof v != 'number' ? v.replace(/<\/?[^>]*>/g, '').replace(/[\r\n]/g, "").replace(/\ +/g, " ").replace(",", "，") : v;
                                str += s + g; 
                            });
                            str += '\t,'; 
                        } else {
                            if (typeof item[prop] != 'object'){
                                let s = typeof item[prop] != 'number' ? item[prop].replace(/<\/?[^>]*>/g, '').replace(/[\r\n]/g, "").replace(/\ +/g, " ").replace(",", "，") : item[prop];
                                str += s + '\t,'; 
                            }
                        }
                    })
                    str += '\n';
                });
            }
            let link = document.createElement("a");
            link.href = 'data:text/' + type + ';charset=utf-8,\ufeff' + encodeURIComponent(str);
            link.download =  "onekeyadmin-" + common.dateTime() + "." + type;
            link.click();
        },
        /**
         * 追加
         */
        pushData(row) {
             let arr = JSON.parse(JSON.stringify(this.drawerForm));
            arr[this.tableTreeProps.parent] = row[this.rowKey];
            this.openData(arr);
        },
         /**
         * 复制
         */
        copyData(row) {
             let arr = JSON.parse(JSON.stringify(row));
            arr[this.rowKey] = "";
            this.openData(arr);
        },
         /**
         * 预览
         */
        previewData(row) {
             window.open(row.url);
        },
        /**
         * 删除数据
         */
        removeData(row = "") {
            let self = this;
            let ids  = row === "" ? common.arrayColumn(self.rows, self.rowKey) : [row[self.rowKey]]; 
            self.$confirm('确定删除数据吗？', '', { confirmButtonText: '确定', cancelButtonText: '取消', type: 'warning'}).then(() => {
                request.post(self.deleteUrl, {'ids': ids}, function(res){
                    if (res.status === 'success') {
                        self.pNode = row === "" ? common.arrayColumn(self.rows, self.tableTreeProps.parent) : [row[self.tableTreeProps.parent]]
                        self.getData();
                        self.$emit('remove-data', res);
                    }
                    self.$notify({ showClose: true, message: res.message, type: res.status});
                });
            }).catch(() => {});
        },
        /**
         * 搜索数据
         */
        searchData() {
            this.search = Object.assign({}, this.search, {page: 1});
        },
        /**
         * 排序改变
         */
        sortChange(val) {
            this.search = Object.assign({}, this.search, {page: 1 ,prop: val.prop , order: val.order});
        },
        /**
         * 分页个数改变时
         */
        pageSizeChange(val) {
            this.search = Object.assign({}, this.search, {page: 1 ,pageSize: val});
        },
        /**
         * 分页改变时
         */
        pageChange(val) {
            this.search = Object.assign({}, this.search, {page: val});
        },
        /**
         * 表格行不可选中
         */
        selectableDisabled(row, index) {
            if (row.disabled === true || row.delete_disabled === true) {
                return false;
            } else {
                return true;
            }
        },
        /**
         * 表格行类名
         */
        tableRowClass({row, rowIndex}) {
            return typeof row.table_row_class !== 'undefined' ? row.table_row_class : '';
        },
        /**
         * 表格懒加载，树形结构需要这样处理
         */
        tableLoad(tree, treeNode, resolve) {
            // 查找下一层数据
            let self = this;
            let children = [];
            let select   = self.rows.indexOf(tree) !== -1;
            self.list.forEach(function (item, index) {
                if (item[self.tableTreeProps.parent] === tree[self.rowKey]) {
                    item.hasChildren = item.children && item.children.length > 0;
                    item.children    = null;
                    children.push(item);
                    self.$refs.table.toggleRowSelection(item, select);
                }
            });
            tree.children = children;
            tree.loadedChildren = true;
            resolve(children);
            self.sNode.set(tree[self.rowKey], { tree, treeNode, resolve });
        },
        /**
         * 表格选中行，树形结构需要这样处理
         */
        selectRow(selection, row) {
            let self = this;
            self.$nextTick(() => {
                let select = self.rows.indexOf(row) !== -1;
                if (typeof row.children !== 'undefined' && row.children != null) {
                    row.children.forEach(function (item, index) {
                        self.$refs.table.toggleRowSelection(item, select);
                        if (typeof item.children !== 'undefined') {
                            self.selectRow([], item);
                        }
                    })
                }
            })
            self.$emit('select-data', self.rows);
        },
        /**
         * 表格选中所有行，树形结构需要这样处理
         */
        selectAll() {
            let self = this;
            self.select = !self.select;
            self.list.forEach(function (item, index){
                if (item.disabled !== true && item.delete_disabled !== true) {
                    self.$refs.table.toggleRowSelection(item, self.select);
                }
            });
            self.$emit('select-data', self.rows);
        },
        /**
         * 表格选中行
         */
        selectionChange(rows) {
            this.rows = rows;
        },
        /**
         * 表单验证
         */
        formRules(is = '', prop = '') {
            let self  = this;
            let rules = {};
            self.colRow.forEach( function (items, index1) {
                if (typeof items.field !== 'undefined') {
                    items.field.forEach(function (item, index2) {
                        if (typeof item.form.rules !== 'undefined') {
                            let test = common.parseJson(common.stringifyJson(item.form.rules));
                            test.forEach(function (rule, index3) {
                                // 新增时不能为空
                                if (typeof rule.saveRequired !== 'undefined' && self.drawerData[self.rowKey] === '') {
                                    test.push({required: true, message: rule.message, trigger: 'blur'});
                                }
                                // 更新时不能为空
                                if (typeof rule.updateRequired !== 'undefined' && self.drawerData[self.rowKey] !== '') {
                                    test.push({required: true, message: rule.message, trigger: 'blur'});
                                }
                                // 关联时不能为空
                                if (typeof rule.relationRequired !== 'undefined') {
                                    let relationRequired = true;
                                    rule.relationRequired.forEach(function(r, i) {
                                        if (typeof r.where != 'undefined') {
                                            if (r.where == 'in') {
                                                if (r.value.indexOf(self.drawerData[r.prop]) === -1) {
                                                    relationRequired = false;
                                                    test.splice(index3, 1);
                                                }
                                            }
                                            if (r.where == '<>') {
                                                if (self.drawerData[r.prop] === r.value) {
                                                    relationRequired = false;
                                                    test.splice(index3, 1);
                                                }
                                            }
                                        } else {
                                            if (self.drawerData[r.prop] !== r.value) {
                                                relationRequired = false;
                                                test.splice(index3, 1);
                                            }
                                        }
                                    })
                                    if (relationRequired) {
                                        test.push({required: true, message: rule.message, trigger: 'blur'});
                                    }
                                }
                            });
                            rules[item.prop] = test;
                        }
                    })
                }
            });
            self.drawerRules = rules;
            self.$emit('input-data', self.drawerData);
            // 特殊类型，重新验证
            if (is == 'el-file-list-select' || is == 'el-file-select' || is == 'el-file-upload') {
                self.$refs.drawerData.validateField(prop);
            }
            // 由于 Form 的校验内置了 async-validator，而它会给每个字段加一个默认的值为 string 的 type 规则，即默认情况下字段必须是字符串型。
            // 但是一般类似于新闻类别选择的这种，其实选择的是类别的id，所以类别就需要你自己设置成number、array等等才可以
            // 如：typevalue: [{ required: true, message: '不能为空', trigger:'change' ,type:'number' }]
        },
        /**
         * 表单提示变量
         */
        formVariable(item) {
            let raw = item.form.is == 'el-editor' ? '|raw' : '';
            return '{$' + this.variable + '.' + item.prop + raw + '}';
        },
        /**
         * 表单内值(远程搜索)
         */
        remoteMethod(query, form) {
            if (query !== '') {
                let self = this;
                request.post(form.remote, {keyword: query}, function(res){
                    if (res.status === 'success') {
                        form.child.value = res.data;
                    } else {
                        self.$notify({ showClose: true, message: res.message, type: res.status});
                    }
                });
            } else {
                form.child.value = [];
            }
        },
        /**
         * 表单内值
         */
        formChildValue(item) {
            let self = this;
            if (typeof item.form.child !== 'undefined') {
                if (typeof item.form.child.value !== 'undefined') {
                    if (item.form.child.value === 'this') {
                        // 本身
                        let list = [];
                        self.list.forEach(function (option, index) {
                            if (self.drawerData[self.rowKey] != option[self.rowKey]) {
                                let json  = {};
                                json['pid'] = typeof option[self.tableTreeProps.parent] === 'undefined' ? 0 : option[self.tableTreeProps.parent];
                                json['value'] = typeof option[self.rowKey] === 'undefined' ? '' : option[self.rowKey];
                                json['label'] = typeof option[self.tableTreeProps.label] === 'undefined' ? '' : option[self.tableTreeProps.label];
                                list.push(json);
                            }
                        })
                        // 树形
                        list = tree.convertString(list, 'pid', 'value');
                        list.unshift({label: '最父级', value: 0});
                        return list;
                    } else {
                        // 传递
                        let list = item.form.child.value;
                        if (typeof item.form.child.props !== 'undefined') {
                            list.forEach(function (option, index) {
                                // 指定
                                if (typeof item.form.child.props.label !== 'undefined') option.label = option[item.form.child.props.label];
                                if (typeof item.form.child.props.value !== 'undefined') option.value = option[item.form.child.props.value];
                            });
                        }
                        return list;
                    }
                }
            }
        },
        /**
         * 表单显示
         */
        formItemShow(item) {
            let self = this;
            let show = true;
            // 不显示
            if (item.form === false) {
                show = false;
            }
            if (typeof item.form.show != 'undefined' && item.form.show === false) {
                show = false;
            }
            // 指定显示字段
            if (this.drawerAppoint != '') {
                if (this.drawerAppoint.indexOf(item.prop) == -1) {
                    show = false;
                }
            } else {
                // 更新时不显示(批量或指定更新不受影响)
                if (self.drawerData[self.rowKey] !== '' && item.form.update === false) {
                    show = false;
                }
            }
            // 主键不显示
            if (item.prop === self.rowKey) {
                show = false;
            }
            // 新增时不显示
            if (self.drawerData[self.rowKey] === '' && item.form.save === false) {
                show = false;
            }
            // 关联不显示
            if (typeof item.form.relation === 'object') {
                item.form.relation.forEach(function(val, index) {
                    if (typeof val.where != 'undefined') {
                        if (val.where == '<>') {
                            if (self.drawerData[val.prop] === val.value) {
                                show = false;
                            }
                        }
                        if (val.where == 'in') {
                            if (val.value.indexOf(self.drawerData[val.prop]) === -1) {
                                show = false;
                            }
                        }
                    } else {
                        if (self.drawerData[val.prop] !== val.value) {
                            show = false;
                        }
                    }
                })
            }
            return show;
        },
        arrayColumn(arr, field) {
            return common.arrayColumn(arr, field);
        }
    },
    watch: {
        search() {
            this.getData();
        },
        drawerData() {
            this.formRules();
        },
        drawer(val) {
            if(!val){
                this.$refs.drawerData.clearValidate();
            }
            this.randId = common.id(16);
        },
        drawerAppoint(val) {
            if (val == '') {
                this.colRowShow = JSON.parse(JSON.stringify(this.colRow));;
            } else {
                let field = [];
                this.field.forEach(function (item, index) {
                    if (typeof item.field == 'undefined') {
                        field.push(item);
                    } else {
                        field = field.concat(item.field);
                    }
                });
                this.colRowShow = [{label: '基础信息', field: field}];
            }
        },
    }
});

/**
 * 树形菜单
 */
Vue.component('el-tree-menu', {
    template: `
    <div class="el-tree-menu">
        <el-checkbox 
            @change="checkedAll()" 
            v-model="isChecked"
            label="全部选中" 
            size="mini" 
            border>
        </el-checkbox>
        <el-checkbox 
            @change="expandAll()"
            v-model="expand"
            label="全部展开" 
            size="mini" 
            border>
        </el-checkbox>
        <el-tree
            node-key="id"
            ref="tree"
            :data="data"
            :props="{children: 'children', label: 'title'}"
            :expand-on-click-node="false"
            @check="checkChange()"
            show-checkbox
            check-on-click-node>
        </el-tree>
    </div>
    `,
    props: {
        value: {
            type: Array,
            default: []
        },
        list: {
            type: Array,
            default: []
        }
    },
    data() {
        return {
            data: tree.convert(this.list),
            isChecked: false,
            expand: false,
        }
    },
    created() {
        let self   = this;
        self.$nextTick(() => {
            var arr    = self.value;
            var newArr = [];
            var item   = '';
            arr.forEach(item=> {
                self.checked(item,self.data,newArr)
            })
            self.$refs.tree.setCheckedKeys(newArr);
        })
    },
    methods: {
        /**
         * 后台数据回显
         */
        checked(id,data,newArr) {
            let self = this;
            data.forEach(item => {
                if(item.id == id){
                    if (typeof item.children === 'undefined') {
                        newArr.push(item.id)
                    }
                }else{
                    if (typeof item.children !== 'undefined') {
                        self.checked(id,item.children,newArr)
                    }
                }
            });
        },
        /**
         * 全选
         */
        checkedAll() {
            let keys = this.isChecked ? common.arrayColumn(this.list, 'id') : [];
            this.$refs.tree.setCheckedKeys(keys);
            this.$emit('input', keys);
        },
        /**
         * 展开
         */
        expandAll() {
            let self = this;
            let keys = common.arrayColumn(self.list, 'id');
            keys.forEach(function(val, key){
                if (self.$refs.tree.store.nodesMap[val].childNodes.length > 0) {
                    self.$refs.tree.store.nodesMap[val].expanded = self.expand;
                }
            })
        },
        /**
         * 状态改变
         */
        checkChange() {
            let roleSel = this.$refs.tree.getCheckedKeys();
            let rolePar = this.$refs.tree.getHalfCheckedKeys();
            this.$emit('input', roleSel.concat(rolePar));
        }
    },
    watch: {
        value(val){
            let self   = this;
            self.$nextTick(() => {
                var arr    = self.value;
                var newArr = [];
                var item   = '';
                arr.forEach(item=> {
                    self.checked(item,self.data,newArr)
                })
                self.$refs.tree.setCheckedKeys(newArr);
            })
        }
    }
})
/**
 * 自定义数组
 */
Vue.component('el-array', {
    template: `
    <div class="el-array" ref="elArray">
        <el-button type="primary" @click="unshifTable()" size="small">新增一行</el-button>
        <el-table :data="table" row-key="id" border>
            <el-table-column v-for="(item, index) in column" :prop="item.field" :key="index" :width="item.type.width">
                <template slot="header" slot-scope="scope">
                    <el-tooltip :content="item.field" placement="top">
                        <span class="el-array-title">{{item.label}}</span>
                    </el-tooltip>
                    <el-tooltip content="删除字段" placement="top">
                        <i class="el-array-remove el-icon-delete" @click="delField(index)"></i>
                    </el-tooltip>
                </template>
                <template slot-scope="scope">
                    <component v-model="scope.row[item.field]" :is="item.type.is" :type="item.type.type"></component>
                </template>
            </el-table-column>
            <el-table-column>
                <template slot="header" slot-scope="scope">
                    <el-button type="primary" @click="dialog = true" size="mini">新增字段</el-button>
                </template>
                <template slot-scope="scope">
                    <a href="javascript:;" @click="removeTable(scope.$index)">删除</a>
                    <a href="javascript:;" @click="pushTable(scope.$index)"">追加</a>
                </template>
            </el-table-column>
        </el-table>
        <el-button type="primary" @click="pushTable()" size="small">新增一行</el-button>
        <el-dialog top="20px" title="列表新增字段" :visible.sync="dialog" :close-on-click-modal="false" width="500px" append-to-body>
            <el-form :model="fieldForm" :rules="rules" ref="fieldForm" label-width="100px" @submit.native.prevent>
                <el-form-item label="字段类型：" prop="type">
                    <el-select style="width:100%" v-model="fieldForm.type" value-key="label" placeholder="请选择字段类型" filterable>
                        <el-option v-for="(item, index) in fieldList" :key="index" :label="item.label" :value="item"></el-option>
                    </el-select>
                </el-form-item>
                <el-form-item label="字段标题：" prop="label">
                    <el-input v-model="fieldForm.label" placeholder="如：内容"></el-input>
                </el-form-item>
                <el-form-item label="字段变量：" prop="field">
                    <el-input v-model="fieldForm.field" placeholder="如：content"></el-input>
                </el-form-item>
                <el-form-item label="字段宽度：" prop="width">
                    <el-input v-model="fieldForm.type.width" placeholder="如：500px"></el-input>
                </el-form-item>
            </el-form>
            <span slot="footer" class="dialog-footer">
                <el-button size="small" type="primary" @click="addField()">确 定</el-button>
                <el-button size="small" @click="dialog = false">取 消</el-button>
            </span>
        </el-dialog>
    </div>
    `,
    props: {
        value: {
            type: Object,
            default: {},
        },
        type: {
            type: String,
            default: '',
        },
    },
    data() {
        var validateRepeatName = (rule, value, callback) => {
            let isExist = common.arrayIndex(this.column, this.fieldForm.field, 'field');
            if(isExist !== -1) {
                callback(new Error('字段变量不能重复！'));
            } else {
                callback();
            }
        }
        return {
            table: [],
            column: {},
            dialog: false,
            fieldForm: {
                label: '',
                field: '',
                type: '',
            },
            rules: {
                label: [
                    { required: true, message: '请填写字段标题', trigger: 'blur' },
                ],
                field: [
                    { required: true, message: '请填写字段变量', trigger: 'blur' },
                    { validator: validateRepeatName, trigger: 'blur' },
                ],
                type: [
                    { required: true, message: '请选择字段类型', trigger: 'blur' },
                ],
            },
            fieldList: [
                {label: '文本', is: 'el-input', width:'200px', value: ''},
                {label: '文本域', is: 'el-input', type: 'textarea', width:'300px', value: ''},
                {label: '编辑器', is: 'el-editor', width:'700px', value: ''},
                {label: '文件上传', is: 'el-file-select', type: 'all', width:'100px', value: ''},
                {label: '链接', is: 'el-link-select', width:'300px', value: {}}
            ],
        }
    },
    created() {
        let bool = JSON.stringify(this.value) == "{}" || this.value == '';
        this.table = bool ?  [] : this.value.table;
        this.column = bool ? [{label: '标题', field: 'title', type: {label: '文本', is: 'el-input', value: ''}}] : this.value.column;
    },
    methods: {
        /**
         * 新增行
         */
        pushTable(index = "") {
            let ob = {};
            this.column.forEach( function (item, index) {
                ob[item.field] = item.type.value;
            });
            index === "" ? this.table.push(ob) : this.table.splice(index + 1, 0, ob);
        },
        unshifTable() {
            let ob = {};
            this.column.forEach( function (item, index) {
                ob[item.field] = item.type.value;
            });
            this.table.unshift(ob);
        },
        /**
         * 删除数组
         */
        removeTable(index){
            this.table.splice(index, 1);
        },
        /**
         * 新增字段
         */
        addField() {
            let self = this;
            self.$refs.fieldForm.validate((valid) => {
                if (valid) {
                    let row = JSON.parse(JSON.stringify(self.fieldForm));
                    self.column.push(row);
                    self.table.forEach(function (item, index) {
                        self.$set(item, row.field, row.type.value);
                    });
                    self.dialog = false;
                    self.$refs.fieldForm.resetFields();
                } else {
                    return false;
                }
            });
        },
        /**
         * 删除字段
         */
        delField(index) {
            let self = this;
            let prop = self.column[index]['field'];
            self.column.splice(index, 1);
            self.table.forEach(function(item, index) {
                self.$delete(item, prop);
            });
        },
    },
    watch: {
        value() {
            let bool = JSON.stringify(this.value) == "{}" || this.value == '';
            this.table = bool ?  [] : this.value.table;
            this.column = bool ? [{label: '标题', field: 'title', type: {label: '文本', is: 'el-input', value: ''}}] : this.value.column;  
        },
        column(){
            this.$emit('input', {column: this.column, table: this.table});
        },
        table() {
            this.$emit('input', {column: this.column, table: this.table});
        },
    }
});
/**
 * 图片水印弹窗
 */
Vue.component('el-watermark', {
    template: `
    <div class="el-watermark">
        <el-checkbox v-model="watermark.open" :false-label="0" :true-label="1" @change="save()" style="margin-right:10px">水印</el-checkbox>
        <el-button
            size="small" 
            icon="el-icon-setting" 
            @click="dialog = true" 
            plain>
            水印设置
        </el-button>
        <el-dialog
            class="el-watermark-dialog"
            top="20px"
            title="设置水印"
            :visible.sync="dialog"
            width="500px"
            :close-on-click-modal="false"
            append-to-body>
            <el-form class="el-watermark-form" label-width="100px" style="height: 550px">
                <el-form-item label="水印类型：">
                    <el-radio-group size="small" v-model="watermark.type">
                        <el-radio label="font">文字</el-radio>
                        <el-radio label="image">图片</el-radio>
                    </el-radio-group>
                </el-form-item>
                <el-form label-width="100px">
                    <template v-if="watermark.type === 'font'">
                        <el-form-item label="文字内容：">
                            <el-input class="el-watermark-text" size="small" v-model="watermark.fontText" placeholder=""></el-input>
                        </el-form-item>
                        <el-form-item label="文字样式：">
                            <el-select size="small" v-model="watermark.fontFamily">
                                <el-option
                                    v-for="(item, index) in fontFamily"
                                    :key="index"
                                    :label="item.label"
                                    :value="item.value">
                                </el-option>
                            </el-select>
                        </el-form-item>
                        <el-form-item label="文字大小：">
                            <el-select size="small" v-model="watermark.fontSize">
                                <el-option
                                    v-for="(item, index) in fontSize"
                                    :key="index"
                                    :value="item">
                                </el-option>
                            </el-select>
                        </el-form-item>
                        <el-form-item label="文字颜色：">
                            <el-color-picker
                                size="small"
                                v-model="watermark.fontColor"
                                :predefine="predefineColors"
                                color-format="hex">
                            </el-color-picker>
                        </el-form-item>
                        <el-form-item class="el-watermark-angle" label="文字倾斜：">
                            <el-slider style="width: 240px" v-model="watermark.fontAngle" show-input></el-slider>
                        </el-form-item>
                    </template>
                    <template v-else>
                        <el-form-item class="el-watermark-angle" label="水印图片：">
                            <el-upload
                                class="el-watermark-uploader"
                                :action="admin_url('file/watermarkUpload')"
                                :show-file-list="false"
                                :on-success="watermarkSuccess"
                                :before-upload="watermarkUpload"
                                accept=".png">
                                <el-image :src="watermark.image"></el-image>
                            </el-upload>
                            只允许.png格式图片
                        </el-form-item>
                    </template>
                </el-form>
                <el-form-item label="水印大小：">
                    <el-radio-group size="small" v-model="watermark.sizeType">
                        <el-radio label="actual">按实际大小</el-radio>
                        <el-radio label="scale">按比例缩放</el-radio>
                    </el-radio-group>
                    <el-select
                        v-if="watermark.sizeType === 'scale'" 
                        size="small" 
                        v-model="watermark.scale">
                        <el-option
                            v-for="(item, index) in scale"
                            :key="index"
                            :label="item + '%'"
                            :value="item">
                        </el-option>
                    </el-select>
                </el-form-item>
                <el-form-item label="水印位置：">
                    <el-col :span="8">
                        <el-select size="small" v-model="watermark.position">
                            <el-option
                                v-for="(item, index) in position"
                                :key="index"
                                :label="item.label"
                                :value="item.value">
                            </el-option>
                        </el-select>
                    </el-col>
                    <el-col :span="15" :offset="1">
                        <div style="display: -webkit-box;display: -moz-box;display: -webkit-flex;display: -ms-flexbox;display: flex;-webkit-box-lines: multiple;-moz-box-lines: multiple;-o-box-lines: multiple;-webkit-flex-wrap: wrap;-ms-flex-wrap: wrap;flex-wrap: wrap;">
                            <div>不透明度：</div>
                            <el-select style="width: 80px" size="small" v-model="watermark.opacity">
                                <el-option
                                    v-for="(item, index) in opacity"
                                    :key="index"
                                    :label="item + '%'"
                                    :value="item">
                                </el-option>
                            </el-select>
                        </div>
                    </el-col>
                </el-form-item>
                <el-form-item label="示例图片：">
                    <div class="el-watermark-example-image">
                        <div slot="error" class="el-watermark-image-slot">
                            <i class="el-icon-picture-outline"></i>
                        </div>
                        <div 
                            class="el-watermark-example" 
                            :class="'el-watermark-position' + watermark.position"
                            :style="{transform: watermarkFontAngle,'-webkit-transform': watermarkFontAngle,'-moz-transform': watermarkFontAngle}">
                            <span
                                v-if="watermark.type === 'font'"
                                :style="{opacity: watermarkOpacity, fontSize: watermarkFontSize, color: watermark.fontColor,fontFamily:watermark.fontFamily}">
                                {{watermark.fontText}}
                            </span>
                            <img
                                v-else
                                style="max-width: 100%;max-height: 100%;"
                                :src="watermark.image"
                                :style="{opacity: watermarkOpacity, width: watermarkImageWidth}">
                        </div>
                    </div>
                </el-form-item>
            </el-form>
            <div slot="footer" class="dialog-footer">
                <el-button @click="dialog = false" size="small">关 闭</el-button>
                <el-button type="primary" @click="save()" :loading="saveLoading" size="small">保 存</el-button>
            </div>
        </el-dialog>
    </div>`,
    props: {
        value: {
            type: Boolean,
            default: false,
        },
    },
    data() {
        return {
            watermark: watermark,
            saveLoading: false,
            dialog: false,
            updateUrl: "file/watermarkConfig",
            scale: ["5","10","15","20","25","30","35","40","45","50"],
            opacity: ["10","20","30","40","50","60","70","80","90","100"],
            fontSize: ["12","14","16","18","20","24","28","32","36","40","48","56","64","72","80"],
            position: [ 
                {label: "左上角", value: "1"},
                {label: "正上方", value: "2"},
                {label: "右上角", value: "3"},
                {label: "左中处", value: "4"},
                {label: "正中处", value: "5"},
                {label: "右中处", value: "6"},
                {label: "左下角", value: "7"},
                {label: "正下方", value: "8"},
                {label: "右下角", value: "9"},
            ],
            fontFamily: [ 
                {label: "默认字体", value: "admin/fonts/FZHTJW.TTF"},
            ],
            predefineColors: [
                '#333333',
                '#ff8c00',
                '#ffd700',
                '#90ee90',
                '#00ced1',
                '#1e90ff',
                '#c71585',
            ],
        }
    },
    computed:{
        watermarkOpacity() {
            return this.watermark.opacity/100;
        },
        watermarkImageWidth() {
            return this.watermark.sizeType === 'actual' ? '': parseInt((this.watermark.scale/100) * 280) + 'px';
        },
        watermarkFontSize() {
            return this.watermark.sizeType === 'actual' ? this.watermark.fontSize + 'px': parseInt((this.watermark.scale/100) * (150/2)) + 'px';
        },
        watermarkFontAngle() {
            var position = '';
            switch(this.watermark.position){
                case ("2"):
                case ("8"):
                    position = 'translateX(-50%)';
                break;
                case ("4"):
                case ("6"):
                    position = 'translateY(-50%)';
                break;  
                case ("5"):
                    position = 'translate(-50%,-50%)';
                break;
            }
            return this.watermark.type === 'font' ? `rotate(-${this.watermark.fontAngle}deg) ${position}` : position;
        },
    },
    methods: {
        /**
         * 保存数据
         */
        save() {
            let self = this;
            if (self.watermark.type !== "image" || self.watermark.image !== "") {
                self.saveLoading = true;
                request.post(self.updateUrl, {'value':self.watermark}, function(res) {
                    self.saveLoading = false;
                    self.dialog  = false;
                    if(res.status !== 'success') {
                        self.$notify({ showClose: true, message: res.message, type: res.status});
                    }
                });
            } else {
                self.$notify({ showClose: true, message: "请上传水印图片", type: "error"});
            }
        },
        /**
         * 设置成功
         */
        watermarkSuccess(res, file) {
            if (res.status === 'success') {
                this.watermark.image = res.url + '?' + Math.floor(Math.random()*10000000);
            } else {
                this.$notify.error(res.message);
            }
        },
        /**
         * 上传水印图
         */
        watermarkUpload(file) {
            const isJPG  = file.type === 'image/png';
            const isLt2M = file.size / 1024 / 1024 < 1;
            if (!isJPG) {
                this.$notify.error('上传水印图片只能是 png 格式!');
            }
            if (!isLt2M) {
                this.$notify.error('上传水印图片大小不能超过 1MB!');
            }
            return isJPG && isLt2M;
        },
        /**
         * 后台链接
         */
        admin_url(link = '') {
            return admin_url(link);
        }
    },
});
/**
 * 文件列表
 */
Vue.component('el-file-list', {
    template: `
    <el-container class="el-file-list el-layout">
        <el-tabs :tab-position="position" v-model="typeIndex" @tab-click="typeChange">
            <el-tab-pane v-for="(item, index) in side" :key="index" :name="String(index)">
                <span slot="label"><i class="iconfont" :class="item.icon"></i>{{item.title}}</span>
            </el-tab-pane>
        </el-tabs>
        <el-container>
            <div class="el-content">
                <div class="el-file-header">
                    <div class="el-file-button">
                        <el-watermark v-show="search.type == 'image' || search.type == '' || search.type == 'all'"></el-watermark>
                        <el-upload 
                            style="margin-right: 10px"
                            v-show="search.type !== 'recycle'" 
                            class="el-upload" 
                            action=""
                            :disabled="uploadLoading"
                            :show-file-list="false"
                            :auto-upload="false"
                            :on-change="fileChange"
                            :accept="accept"
                            multiple>
                            <el-button :disabled="uploadLoading" size="small" icon="el-icon-upload2" title="支持上传超大文件断点续传" plain>{{uploadLoading ? '上传中...' : '上传'}}</el-button>
                        </el-upload>
                        <el-button v-show="search.type !== 'recycle'" size="small" icon="el-icon-delete" :disabled="rows.length === 0" @click="recovery()" plain>放入回收站</el-button>
                        <el-button v-show="search.type === 'recycle'" size="small" icon="el-icon-delete" @click="emptyTrash()" plain>清空</el-button>
                        <el-button v-show="search.type === 'recycle'" size="small" icon="el-icon-wallet" :disabled="rows.length === 0" @click="reduction()" plain>还原</el-button>
                        <el-button v-show="search.type === 'recycle'" size="small" icon="el-icon-delete" :disabled="rows.length === 0" @click="remove()" plain>删除</el-button>
                    </div>
                    <div class="el-file-search">
                        <el-input size="small" placeholder="请输入文件名/文件链接" v-model="search.keyword" @keyup.enter.native="keywordChange()">
                            <el-button slot="append" icon="el-icon-search" @click="keywordChange()"></el-button>
                        </el-input>
                        <el-button-group>
                            <el-button size="small" :type="show == 'flat' ? 'primary' : ''" icon="iconfont icon-caidan1" @click="show = 'flat'"></el-button>
                            <el-button size="small" :type="show == 'table' ? 'primary' : ''" icon="iconfont icon-caidan2" @click="show = 'table'"></el-button>
                        </el-button-group>
                    </div>
                </div>
                <div class="theader">
                    <div class="file">
                        <el-checkbox v-model="checkAll">
                            <template v-if="rows.length > 0">已选中{{rows.length}}个</template>
                            <template v-else><span class="title">文件名</span></template>
                        </el-checkbox>
                        <span class="caret-wrapper" :class="{active: search.prop === 'title'}">
                            <i class="sort-caret asc" :class="{active: search.order === 'asc'}" @click="orderChange('title', 'asc')"></i>
                            <i class="sort-caret desc" :class="{active: search.order === 'desc'}" @click="orderChange('title', 'desc')"></i>
                        </span>
                    </div>
                    <template v-if="show == 'table'">
                        <div class="size">
                            <span class="title" @click="orderChange('size', search.order === 'asc' ? 'desc' : 'asc')">大小</span>
                            <span class="caret-wrapper" :class="{active: search.prop === 'size'}">
                                <i class="sort-caret asc" :class="{active: search.order === 'asc'}" @click="orderChange('size', 'asc')"></i>
                                <i class="sort-caret desc" :class="{active: search.order === 'desc'}" @click="orderChange('size', 'desc')"></i>
                            </span>
                        </div>
                        <div class="time">
                            <span class="title" @click="orderChange('create_time', search.order === 'asc' ? 'desc' : 'asc')">修改日期</span>
                            <span class="caret-wrapper" :class="{active: search.prop === 'create_time'}">
                                <i class="sort-caret asc" :class="{active: search.order === 'asc'}" @click="orderChange('create_time', 'asc')"></i>
                                <i class="sort-caret desc" :class="{active: search.order === 'desc'}" @click="orderChange('create_time', 'desc')"></i>
                            </span>
                        </div>
                    </template>
                </div>
                <div class="list" :class="show" v-infinite-scroll="getData" :infinite-scroll-disabled="disabled" scroll-container>
                    <el-empty v-if="table.length === 0 && loading === false" description="空空如也~"></el-empty>
                    <template v-else>
                        <div class="item" v-for="(item, index) in table" :key="index" @click="checked(item)" :class="{active: item.check}">
                            <div class="file">
                                <el-checkbox v-if="item.status !== 100" v-model="item.check" disabled></el-checkbox>
                                <div v-else style="width: 14px;display: inline-block;"></div>
                                <div class="cover">
                                    <el-image v-if="item.status !== 100" class="image" :src="item.cover"></el-image>
                                    <el-progress v-else type="circle" :percentage="item.percentage"></el-progress>
                                </div>
                                <div class="title" @click.stop="preview(item, index)">
                                    <template v-if="item.status !== 100">
                                        <el-input v-if="item.rename" v-model="item.ctitle" :ref="'title' + index" @keyup.enter.native="renameConfirm(item)">
                                            <template slot="append">
                                                <i class="el-icon-check" @click.stop="renameConfirm(item)"></i>
                                                <i class="el-icon-close" @click.stop="renameClose()"></i>
                                            </template>
                                        </el-input>
                                        <span v-else>{{item.title}}</span>
                                        <el-image 
                                            v-show="false" 
                                            :ref="'preview' + index" 
                                            :src="item.url" 
                                            :preview-src-list="previewImages">
                                        </el-image>
                                    </template>
                                    <span v-else v-html="item.title"></span>
                                </div>
                                <div class="operation" v-if="item.status !== 100 && item.rename === false">
                                    <span v-show="search.type !== 'recycle'" @click.stop="renameClick(item, index)" class="el-icon-edit" title="重命名"></span>
                                    <span @click.stop="downloadClick(item)" class="el-icon-download" title="下载"></span>
                                    <span v-show="search.type !== 'recycle'" @click.stop="recovery(item)" class="el-icon-delete" title="删除"></span>
                                    <span v-show="search.type === 'recycle'" @click.stop="reduction(item)" class="el-icon-folder-checked" title="还原文件"></span>
                                    <span v-show="search.type === 'recycle'" @click.stop="remove(item)" class="el-icon-delete" title="彻底删除"></span>
                                </div>
                            </div>
                            <div class="size">{{fileSize(item.size)}}</div>
                            <div class="time">{{item.create_time}}</div>
                        </div>
                    </template>
                    <p class="el-bottom-loading" v-if="loading"><i class="el-icon-loading"></i><span>拼命加载中...</span></p>
                    <p class="el-bottom-loading" v-if="noMore && count > 0"><span>主人，我已经到底啦！</span></p>
                </div>
            </div>
        </el-container>
    </el-container>
    `,
    props: {
        value: {
            type: Array,
            default: () => {
                return [];
            },
        },
        limit: {
            type: Number,
            default: -1,
        },
        type: {
            type: String,
            default: '',
        },
        tab: {
            type: String,
            default: '0',
        },
    },
    data() {
        return {
            position: document.body.clientWidth > 768 ? 'left' : 'top',
            show: document.body.clientWidth > 768 ? 'table' : 'flat',
            list: [],
            uploadKey: 0,
            uploadList: [],
            uploadLoading: false,
            bytesPerPiece: 1024 * 1024 * 2, // 每个文件切片大小定为2MB
            rows: this.value,
            loading: false,
            checkAll: false,
            indexUrl: "file/index",
            delUrl: "file/delete",
            editUrl: "file/update",
            recoveryUrl: "file/recovery",
            reductionUrl: "file/reduction",
            emptyTrashUrl: "file/emptyTrash",
            search: {
                keyword: '',
                type: '',
                page: 1,
                prop: 'create_time',
                order: 'desc',
            },
            typeIndex: this.tab,
            typeList: [
                {name: "all", title: "全部", icon: 'icon-quanbuwenjian', accept: '.png,.jpg,.jpeg,.bmp,.gif,.ico,.mp4,.mp3,.docx,.doc,.swf,.psd,.css,.js,.html,.exe,.dll,.zip,.rar,.ppt,.pdf,.xlsx,.xls,.txt,.torrent,.dwt,.sql,.svg'},
                {name: "image", title: "图片", icon: 'icon-xingzhuang-tupian', accept: '.png,.jpg,.jpeg,.bmp,.gif,.ico'},
                {name: "video", title: "视频", icon: 'icon-shipin', accept: '.mp4'},
                {name: "audio", title: "音乐", icon: 'icon-yinle', accept: '.mp3'},
                {name: "word",  title: "文档", icon: 'icon-icon_shiyongwendang', accept: '.docx,.doc'},
                {name: "other", title: "其它", icon: 'icon-wenjian', accept: '.swf,.psd,.css,.js,.html,.exe,.dll,.zip,.rar,.ppt,.pdf,.xlsx,.xls,.txt,.torrent,.dwt,.sql,.svg'},
            ],
            propList: [
                { title: "按修改时间排序", name: "create_time"},
                { title: "按文件大小排序", name: "size"},
                { title: "按文件名排序", name: "title"},
            ],
        }
    },
    mounted () {
        let self = this;
        window.onresize = function(){
            self.position = document.body.clientWidth > 768 ? 'left' : 'top';
            self.show     = document.body.clientWidth > 768 ? 'table' : 'flat';
        };
    },
    computed:{
        table() {
            let self = this;
            let list = this.uploadList.concat(this.list);
            list.forEach( function (item, key) {
                let index  = common.arrayIndex(self.rows, item.id, 'id');
                item.check = index === -1 ? false : true;
            })
            return list;
        },
        side() {
            switch(this.type) {
                case '':
                    let side = this.typeList;
                    side.push({name: 'recycle', title: "回收站", icon: 'icon-huishouzhan'});
                    return side;
                break;
                case 'all':
                    return this.typeList;
                break;
                default:
                    let index = common.arrayIndex(this.typeList, this.type, 'name');
                    return [this.typeList[index]];
            }
        },
        name() {
            return this.side[this.typeIndex]['name'];
        },
        accept() {
            return this.side[this.typeIndex]['accept'];
        },
        noMore() {
            return this.count <= this.list.length;
        },
        disabled() {
            return this.loading || this.noMore;
        },
        previewImages() {
            let arr = [];
            this.table.forEach( function (item, index) {
                if (item.type === 'image') {
                    arr.push(item.url);
                }
            })
            return arr;
        },
    },
    created() {
        this.search.type = this.side[this.typeIndex]['name'];
    },
    methods: {
        /**
         * 选择文件
         */
        fileChange(file) {
            if (this.uploadLoading == false) {
                this.list       = this.uploadList.concat(this.list);
                this.uploadKey  = 0;
                this.uploadList = [];
            }
            this.uploadList.push({
                id: '',
                size: file.raw.size,
                name: file.raw.name,
                title: file.raw.name + '<span style="color: #409EFF">（等待上传）</span>',
                status: 100,
                percentage: 0,
                file: file.raw,
                index: 0,
                start: 0,
                totalPieces: Math.ceil(file.raw.size / this.bytesPerPiece),
                create_time: common.dateTime()
            });
            if (this.uploadLoading == false) {
                this.upload(this.uploadKey);
            }
        },
        /**
         * 上传文件
         */
        upload(uploadKey) {
            let self  = this;
            self.uploadLoading = true;
            let blob  = self.uploadList[uploadKey];
            let start = blob.index*self.bytesPerPiece;
            blob.index++;
            let end   = blob.index*self.bytesPerPiece;
            if (end > blob.size) {
                end = blob.size;
            }
            let formData = new FormData();
            formData.append("id", blob.id);
            formData.append("size", blob.size);
            formData.append("file", blob.file.slice(start, end), blob.name);
            formData.append("index", blob.index);
            formData.append("count", blob.totalPieces);
            formData.append("create_time", blob.create_time);
            formData.append("start", start);
            formData.append("end",   end);
            formData.append("theme", theme);
            $.ajax({
                id: '',
                url: admin_url('file/upload'),
                type: 'post',
                data: formData,
                cache:false,
                contentType:false,
                processData:false,
                xhr: function() {
                    var xhr = new XMLHttpRequest();
                    xhr.upload.addEventListener('progress', function (e) {
                        let percentage = Math.round(
                            (e.loaded * blob.index * 100) / blob.size
                        );
                        if (self.uploadList[uploadKey].percentage < percentage) {
                            self.uploadList[uploadKey].title = self.uploadList[uploadKey].name + '<span style="color: #409EFF">（上传中...）</span>';
                            self.uploadList[uploadKey].percentage = percentage > 100 ? 100 : percentage;
                        }
                    })
                    return xhr;
                },
                success:function(res) {
                    if (res.status == 'success') {
                        if (blob.index == blob.totalPieces) {
                            self.uploadList[uploadKey].title = self.uploadList[uploadKey].name + '<span style="color: #67C23A">（上传成功）</span>';
                            self.uploadList[uploadKey].percentage = 100;
                            self.$set(self.uploadList, uploadKey, self.initData(res.data));
                            // 执行下一个文件
                            if (self.uploadList.length - 1 > self.uploadKey) {
                                self.uploadKey++;
                                self.upload(self.uploadKey);
                            } else {
                                // 上传完毕
                                self.uploadKey++;
                                self.uploadLoading = false;
                            }
                        } else {
                            // 是否断点续传
                            self.uploadList[uploadKey]['id']    = res.id; 
                            self.uploadList[uploadKey]['index'] = res.index; 
                            // 执行下个分片
                            self.upload(uploadKey);
                        }
                    } else {
                        // 执行下一个文件
                        if (self.uploadList.length - 1 > self.uploadKey) {
                            self.uploadKey++;
                            self.upload(self.uploadKey);
                        } else {
                            // 上传完毕
                            self.uploadKey++;
                            self.uploadLoading = false;
                        }
                        self.uploadList[uploadKey].title = self.uploadList[uploadKey].name + '<span style="color: #F56C6C">（上传失败）</span>';
                    }
                },
                error:function(res) {
                    // 执行下一个文件
                    if (self.uploadList.length - 1 > self.uploadKey) {
                        self.uploadKey++;
                        self.upload(self.uploadKey);
                    } else {
                        // 上传完毕
                        self.uploadKey++;
                        self.uploadLoading = false;
                    }
                    self.uploadList[uploadKey].title = self.uploadList[uploadKey].name + '<span style="color: #F56C6C">（上传失败）</span>';
                }
            })
        },
        /**
         * 获取数据
         */
        getData() {
            let self     = this;
            self.loading = true;
            request.post(self.indexUrl, self.search, function(res) {
                res.data.forEach(function (item, index) {
                    item = self.initData(item);
                })
                self.search.page++;
                self.loading = false;
                self.list    = self.list.concat(res.data);
                self.count   = res.count;
            });
        },
        /**
         * 初始化数据
         */
        initData(item) {
            let cover    = '/admin/images/filecover/';
            let type     = file.type(item.url)['name'];
            let name     = item.url.substring(item.url.lastIndexOf('/') + 1, item.url.lastIndexOf('.'));
            item['type'] = type;
            if (type === 'image') {
                let suffix = item.url.substring(item.url.lastIndexOf('.') + 1);
                if (suffix != 'gif' && suffix != 'ico') {
                    item['cover']  = item.url.replace(name, name + '100x100');
                } else {
                    item['cover']  = item.url;
                }
            } else {
                item['cover']  = cover + type + '.png';
            }
            item['rename'] = false;
            item['ctitle'] = item.title;
            return item;
        },
        /**
         * 选择
         */
        checked(item) {
            if (typeof item.percentage == 'undefined') {
                let index = common.arrayIndex(this.rows, item.id, 'id');
                if (index === -1) {
                    this.rows.push(item);
                } else {
                    this.rows.splice(index , 1);
                }
            }
        },
        /**
         * 类型
         */
        typeChange() {
            this.search = Object.assign({}, this.search, {page: 1, type: this.name, keyword: ''});
        },
        /**
         * 刷新
         */ 
        refresh() {
            this.search  = Object.assign({}, this.search, {page: 1, keyword: ''});
        },
        /**
         * 搜索
         */
        keywordChange() {
            this.search  = Object.assign({}, this.search, {page: 1});
        },
        /**
         * 名称排序
         */
        propChange(prop) {
            this.search  = Object.assign({}, this.search, {page: 1, prop: prop, order: 'desc'});
        },
        /**
         * 升降排序
         */
        orderChange(prop, order) {
            this.search  = Object.assign({}, this.search, {page: 1, prop: prop, order: order});
        },
        /**
         * 预览
         */
        preview(item, index) {
            if (item.rename === false) {
                let type = file.type(item.url);
                if (type.name === 'image') {
                    this.$refs['preview' + index][0].clickHandler();
                } else {
                    window.open(item.url);
                }
            }
        },
        /**
         * 点击下载
         */
        downloadClick(item) {
            window.open(admin_url('file/download', {title: item.title, url: item.url}));
        },
        /**
         * 点击重命名
         */
        renameClick(item, index) {
            let self = this;
            self.renameClose();
            item.rename = true;
            self.$nextTick(()=>{
                self.$refs[`title${index}`][0].focus();
                self.$refs[`title${index}`][0].select();
            });
        },
        /**
         * 取消重命名
         */
        renameClose() {
            this.table.forEach(function(item) {
                item.ctitle = item.title;
                item.rename = false;
            })
        },
        /**
         * 确定重命名
         */
        renameConfirm(item) {
            let self = this;
            request.post(self.editUrl, {id: item.id, title: item.ctitle}, function(res) {
                if (res.status === 'success') {
                    item.title = item.ctitle;
                    self.renameClose();
                } else {
                    self.$notify({ showClose: true, message: res.message, type: res.status});
                }
            });
        },
        /**
         * 放入回收站
         */
        recovery(item = "") {
            let self = this;
            let rows = item === "" ? self.rows : [item];
            let ids  = common.arrayColumn(rows, 'id');
            self.$confirm('将文件放入回收站', '', {
                confirmButtonText: '确定',
                cancelButtonText: '取消',
                type: 'warning'
            }).then(() => {
                request.post(self.recoveryUrl, {ids: ids}, function(res){
                    if(res.status === 'success') {
                        self.refresh();
                    } else {
                        self.$notify({ showClose: true, message: res.message, type: res.status});
                    }
                });
            }).catch(() => {});
        },
        /**
         * 彻底删除
         */
        remove(item = "") {
            let self = this;
            let rows = item === "" ? self.rows : [item];
            let ids  = common.arrayColumn(rows, 'id');
            self.$confirm('彻底删除后文件无法找回，确定删除吗？', '', {
                confirmButtonText: '确定',
                cancelButtonText: '取消',
                type: 'error'
            }).then(() => {
                request.post(self.delUrl, {ids: ids}, function(res){
                    if(res.status === 'success') {
                        self.refresh();
                    } else {
                        self.$notify({ showClose: true, message: res.message, type: res.status});
                    }
                });
            }).catch(() => {});
        },
        /**
         * 还原文件
         */
        reduction(item = "") {
            let self = this;
            let rows = item === "" ? self.rows : [item];
            let ids  = common.arrayColumn(rows, 'id');
            request.post(self.reductionUrl, {ids: ids}, function(res){
                if(res.status === 'success') {
                    self.refresh();
                } else {
                    self.$notify({ showClose: true, message: res.message, type: res.status});
                }
            });
        },
        /**
         * 清空回收站
         */
        emptyTrash() {
            let self       = this;
            self.$confirm('清空回收站后所有文件无法找回，确定清空吗？', '', {
                confirmButtonText: '确定',
                cancelButtonText: '取消',
                type: 'error'
            }).then(() => {
                request.post(self.emptyTrashUrl, {}, function(res){
                    if(res.status === 'success') {
                        self.refresh();
                    } else {
                        self.$notify({ showClose: true, message: res.message, type: res.status});
                    }
                });
            }).catch(() => {});
        },
        fileSize(size) {
            return file.size(size);
        }
    },
    watch: {
        rows(val) {
            this.$emit('input', val);
        },
        value(val) {
            this.rows = val;
        },
        search(val) {
            this.uploadKey  = 0;
            this.checkAll   = false;
            this.uploadList = [];
            this.list       = [];
            this.rows       = [];
            this.getData();
        },
        checkAll(val) {
            this.rows = val ? JSON.parse(JSON.stringify(this.table)) : [];
        },
        typeIndex(val) {
            // 父级
		    let path = parent.parentVm.path.split('#');
		    parent.parentVm.path = path[0] + '#' + val;
        },
    }
});

/**
 * 文件弹窗选择
 */
Vue.component('el-file-dialog', {
    template: `
    <el-dialog v-if="dialog" top="20px" width="925px" class="el-file-dialog" title="文件选择" :visible.sync="dialog" :close-on-click-modal="false" append-to-body>
        <el-file-list :type="type" v-model="selected"></el-file-list>
        <div class="el-file-selected">
            <div class="el-limit">
                <template v-if="limit === 0">无限制</template>
                <template v-else>限制<span>&nbsp;{{selected.length}}&nbsp;/&nbsp;{{limit}}</span></template>
            </div>
            <div class="el-list-selected">
                <div class="el-item-selected" title="查看大图" v-for="(item, index) in selected" :key="index" @click="preview(item, index)">
                    <img :src="item.cover"/>
                    <el-image :src="item.url" :ref="'preview' + index" :preview-src-list="previewImages" style="display:none"></el-image>
                    <i class="el-icon-close el-file-remove" @click.stop="remove(index)"></i>
                </div>
            </div>
        </div>
        <span slot="footer" class="dialog-footer">
            <el-button @click="dialog = false" size="small">取 消</el-button>
            <el-button type="primary" @click="success" size="small">确 定</el-button>
        </span>
    </el-dialog>`,
    props: {
        list: {
            type: Array,
            default: [],
        },
        type: {
            type: String,
            default: 'image',
        },
        limit:{
            type: Number,
            default: 1, 
        },
        value: {
            type: Boolean,
            default: false,
        },
    },
    data() {
        return {
            dialog: this.value,
            selected: JSON.parse(JSON.stringify(this.list)),
        }
    },
    computed: {
        previewImages() {
            let arr = [];
            this.selected.forEach( function (item, index) {
                if (item.type === 'image') {
                    arr.push(item.url);
                }
            })
            return arr;
        },
    },
    methods: {
        preview(item, index) {
            if (item.rename === false) {
                let type = file.type(item.url);
                if (type.name === 'image') {
                    this.$refs['preview' + index][0].clickHandler();
                } else {
                    window.open(item.url);
                }
            }
        },
        success() {
            this.dialog = false;
            this.$emit('success-selected',this.selected);
        },
        remove(index) {
            this.selected.splice(index, 1);
        },
    },
    watch: {
        value(val) {
            this.dialog = val;
        },
        list(val) {
            this.selected = JSON.parse(JSON.stringify(val));
        },
        dialog(val) {
            this.$emit('input', val);
            if (val) {
                this.selected = JSON.parse(JSON.stringify(this.list));
            }
        },
        selected(val) {
            if (val.length > this.limit && this.limit != 0) {
                this.selected = val.slice(val.length - this.limit);
            }
        },
    }
});
/**
 * 多文件选择
 */
Vue.component('el-file-list-select', {
    template: `
    <div class="el-file-list-select" :class="{href: href}">
        <div class="upload" @click="dialog = true" >
            <i class="el-icon-plus el-push"></i>
        </div>
        <draggable class="draggable" v-model="list" v-bind="draggable">
            <div class="item" v-for="(item, index) in list" :key="index" title="拖动排序">
                <i class="el-icon-close el-remove"  @click="list.splice(index, 1)"></i>
                <el-file-select v-model="item.url" :remove="false"></el-file-select>
                <div v-if="href">
                    <div><el-input size="mini" v-model="item.title"></el-input></div>
                    <div><el-link-select size="mini" v-model="item.href"></el-link-select></div>
                </div>
            </div>
        </draggable>
        <el-file-dialog v-model="dialog" :list="list" :type="type" :limit="0" @success-selected="fileSuccess($event)"></el-file-dialog>
    </div>
    `,
    props: {
        value: {
            default: [],
        },
        type: {
            type: String,
            default: 'image'
        },        
        href: {
            type: Boolean,
            default: false
        },
    },
    data() {
        return {
            list: this.value,
            dialog: false,
            draggable: {
                handle: '.item',
                animation: 300,
                forceFallback: true,
                delay: 200,
            }
        }
    },
    created() {
        if (typeof this.list != 'object') {
            this.list = [];
        }
    },
    methods: {
        /**
         * 文件选择成功
         */
        fileSuccess(list) {
            this.list = list;
        },
    },
    watch: {
        value(val) {
            this.list = val;
        },
        list(val) {
            this.$emit('input', val);
        },
    }
});
/**
 * 文件上传器
 */
Vue.component('el-file-upload', {
    template: `
    <div class="el-file-upload">
        <div>
            <el-upload 
                list-type="picture-card"
                class="el-upload" 
                :action="admin_url('api/upload')"
                :accept="type"
                :multiple="multiple"
                :show-file-list="false"
                :on-success="handleSuccess">
                <i class="el-icon-plus"></i>
            </el-upload>
            <ul class="el-upload-list el-upload-list--picture-card">
                <draggable class="draggable" v-model="fileList" v-bind="draggable">
                    <li class="el-upload-list__item is-success" v-for="(item, index) in fileList" :key="index">
                        <el-image 
                            class="el-upload-list__item-thumbnail" 
                            :ref="'preview' + index" 
                            :src="cover(item)" 
                            :preview-src-list="fileList">
                        </el-image>
                        <span class="el-upload-list__item-actions">
                            <span class="el-upload-list__item-preview" @click="handlePreview(index, item)"><i class="el-icon-zoom-in"></i></span>
                            <span v-if="limit > 1" class="el-upload-list__item-delete"><i class="el-icon-rank"></i></span>
                            <span class="el-upload-list__item-delete" @click="handleRemove(index)"><i class="el-icon-delete"></i></span>
                        </span>
                    </li>
                </draggable>
            </ul>
        </div>
        <div style="margin:0" class="el-upload__tip" slot="tip">仅支持上传{{type}}格式，上传限制{{fileList.length}}/{{limit}}</div>
    </div>`,
    props: {
        value: {
            default: '',
        },
        type: {
            type: String,
            default: '',
        },
        multiple: {
            type: Boolean,
            default: false,
        },
        limit: {
            type: Number,
            default: 1,
        },
    },
    data() {
        return {
            fileList: [],
            draggable: {
                handle: '.el-icon-rank',
                animation: 300,
            }
        }
    },
    created() {
        if (typeof(this.value) == 'string') {
            this.fileList = this.value == '' ? [] : this.value.split(',');
        }
    },
    methods: {
        // 获取封面
        cover(item) {
            let type = file.type(item);
            return type.name === 'image' ? item : '/admin/images/filecover/' + type.name + '.png';
        },
        // 链接
        admin_url(link) {
            return admin_url(link);
        },
        // 预览
        handlePreview(index, item) {
            let type = file.type(item);
            type.name === 'image' ? this.$refs['preview' + index][0].clickHandler() : window.open(item);
        },
        // 删除
        handleRemove(index) {
            this.fileList.splice(index, 1);
        },
        // 上传成功
        handleSuccess(res) {
            if (res.status == 'success') {
                if (this.limit <= this.fileList.length) {
                    this.fileList.splice(this.fileList.length - 1, 1);
                }
                this.fileList.push(res.data.url);
            }
        },
    },
    watch: {
        fileList(v) {
            this.$emit('input', v.join(','));
        },
    }
});
/**
 * 单文件选择
 */
Vue.component('el-file-select', {
    template: `
    <div class="el-file-select">
        <div class="el-file-select-warp" :style="{height: size + 'px', width: size + 'px'}">
            <el-image ref="preview" :preview-src-list="[cover]" :src="cover" :style="{lineHeight: size + 'px'}">
                <div slot="error" class="image-slot">
                    <img class="error-image" src="/admin/images/error.png"/>
                </div>
            </el-image>
            <i v-if="remove" class="el-icon-close el-remove" @click="file = ''"></i>
            <div class="tips">
                <div class="icon">
                    <i :style="iconFont" class="el-icon-folder-opened" title="从资源库中选择" @click="dialog = true"></i>
                    <i :style="iconFont" class="el-icon-search" title="查看大图" @click="preview()"></i>
                </div>
            </div>
        </div>
        <el-file-dialog title="文件选择" v-model="dialog" :list="[]" :type="type" :limit="1" @success-selected="success($event)"></el-file-dialog>
    </div>`,
    props: {
        value: {
            type: String,
            default: ''
        },
        type: {
            type: String,
            default: 'image'
        },
        size: {
            type: Number,
            default: 50,
        },
        remove: {
            type: Boolean,
            default: true,
        }
    },
    data() {
        return {
            file: this.value,
            dialog: false,
        }
    },
    computed: {
        iconFont() {
            let size = Math.round(this.size * 0.7 / 2);
            size = size > 12 ? size : 12;
            size = size > 20 ? 20 : size;
            return {fontSize: size + 'px'};
        },
        fileType() {
            let type = file.type(this.file);
            return type.name;
        },
        cover() {
            if (this.type === 'image') {
                return this.file;
            } else {
                return this.fileType === 'image' ? this.file : '/admin/images/filecover/' + this.fileType + '.png';
            }
        },
    },
    methods: {
        preview() {
            if (this.type === 'image') {
                this.$refs.preview.clickHandler();
            } else {
                if (this.fileType === 'image') {
                    this.$refs.preview.clickHandler();
                } else {
                    window.open(this.file);
                }
            }
        },
        success(list) {
            this.file = list[0]['url'];
        },
    },
    watch: {
        file(val) {
            this.$emit('input', val);
            this.$emit('change', val);
        },
        value(val) {
            this.file = val;
        }
    }
});
/**
 * 设置链接
 */
Vue.component('el-link-select', {
    template:`
    <div class="el-link-select">
        <el-input :value="title" :size="size" @focus="dialog = true">
            <i slot="suffix" class="el-input__icon el-icon-arrow-right"></i>
        </el-input>
        <el-dialog v-if="dialog" class="el-link-dialog" top="20px" title="设置链接" width="540px" :visible.sync="dialog" :close-on-click-modal="false" append-to-body>
            <el-form :model="valueForm" ref="valueForm" :rules="rules" label-width="100px" :validate-on-rule-change="false" @submit.native.prevent>
                <el-form-item label="链接类型：" prop="type">
                    <el-radio-group v-model="linkForm.type" @change="typeChange()">
                        <el-radio v-for="(item, index) in typeList" :label="String(index)" :key="index">
                            {{item}}
                        </el-radio>
                    </el-radio-group>
                </el-form-item>
                <template v-if="linkForm.type == 1">
                <el-form-item label="链接：" prop="url">
                    <el-input v-model="valueForm.url" placeholder="请填写链接"></el-input>
                </el-form-item>
                </template>
                <template v-if="linkForm.type == 2">
                    <el-form-item label="类型：" prop="table">
                        <el-select placeholder="请选择类型" v-model="valueForm.table" @change="detailsListSearch()" filterable>
                            <el-option v-for="(item, index) in tableList" :label="item.title" :value="item.table"></el-option>
                        </el-select>
                    </el-form-item>
                    <el-form-item v-if="valueForm.table != ''" label="链接：" prop="details">
                        <el-select value-key="id" placeholder="请选择链接，输入标题远程搜索" v-model="valueForm.details" reserve-keyword filterable remote :remote-method="detailsListSearch">
                            <el-option v-for="(item, index) in detailsList" :key="index" :label="item.title" :value="item"></el-option>
                        </el-select>
                    </el-form-item>
                </template>
                <template v-if="linkForm.type == 3">
                    <el-form-item label="分类：" prop="catalog">
                        <el-select v-model="valueForm.catalog" value-key="id" placeholder="请选择分类" filterable>
                            <el-option v-for="(item, index) in catalogList" :key="index" :label="item.title" :value="item">
                                <span v-html="item.treeString"></span>
                                {{ item.title }}
                            </el-option>
                        </el-select>
                    </el-form-item>
                    <el-form-item label="锚点：" prop="anchor">
                        <el-input v-model="valueForm.anchor" placeholder="如：#about"></el-input>
                    </el-form-item>
                </template>
            </el-form>
            <span slot="footer" class="dialog-footer">
                <el-button size="small" type="primary" @click="determine">确定</el-button>
                <el-button size="small" @click="dialog = false">取消</el-button>
            </span>
        </el-dialog>
    </div>`,
    props: {
        value: {},
        size: {
            type: String,
            default: '',
        },
    },
    data() {
        return {
            url: "api/link",
            catalogUrl: "api/catalog",
            linkForm: {},
            linkDefault:{
                type: "0",
                value: [
                    {},
                    {url: ""},
                    {table: "", details: ""},
                    {catalog: "", anchor: ""},
                ],
            },
            dialog: false,
            typeList: ['无', '自定义链接', '详情链接', '分类链接'],
            tableList: [],
            detailsList: [],
            catalogList: [],
            title: "",
        }
    },
    created() {
        this.linkForm = JSON.stringify(this.value) === "{}" || this.value.length === 0 ? this.linkDefault : this.value;
        this.init();
    },
    computed: {
        valueForm() {
            return this.linkForm['value'][this.linkForm.type];
        },
        rules() {
            let rules = {};
            switch(this.linkForm.type){
                case ("1"):
                    rules = {}
                    break;
                case ("2"):
                    rules = {
                        table: [
                            { required: true, message: '请选择类型', trigger: 'blur' },
                        ],
                        details: [
                            { required: true, message: '请选择链接', trigger: 'blur' },
                        ],
                    }
                    break;
                case ("3"):
                    rules = {
                        catalog: [ { required: true, message: '请选择分类', trigger: 'blur' } ]
                    }
                    break;
            }
            return rules;
        },
    },
    methods: {
        /**
         * 初始化
         */
        init() {
            this.catalogListSearch();
            this.tableListSearch();
            this.detailsListSearch();
            this.titleSearch();
        },
        /**
         * 标题
         */
        titleSearch() {
            switch(this.linkForm.type){
                case ("0"):
                case ("1"):
                    this.title = this.valueForm['url'];
                    break;
                case ("2"):
                    this.title = this.valueForm['details'] !== '' ? this.valueForm['details']['title'] : '';
                    break;
                case ("3"):
                    let title = this.valueForm['catalog'] !== '' ? this.valueForm['catalog']['title'] : '';
                    this.title = title + this.valueForm['anchor'];
                    break;
            }
        },
        /**
         * 类型改变
         */
        typeChange() {
            this.$refs.valueForm.clearValidate();
            this.init();
        },
        /**
         * 分类搜索
         */
        catalogListSearch() {
            let self = this;
            if (this.linkForm.type == '3' && this.catalogList.length == 0) {
                request.post(self.catalogUrl, {theme: theme}, function(res) {
                    self.catalogList = tree.convertString(res.data);
                });
            }
        },
        /**
         * 表搜索
         */
        tableListSearch() {
            let self = this;
            if (this.linkForm.type == '2' && this.tableList.length == 0) {
                request.post(self.url, {}, function(res) {
                    self.tableList = res.data;
                });
            }
        },
        /**
         * 表详情搜索
         */
        detailsListSearch(keyword = "") {
            let self = this;
            if (this.linkForm.type == '2' && self.valueForm.table != '') {
                request.post(self.url, {table: self.valueForm.table, keyword: keyword}, function(res) {
                    self.detailsList = res.data;
                });
            }
        },
        /**
         * 确定
         */
        determine() {
            this.$refs.valueForm.validate((valid) => {
                if (valid) {
                    this.titleSearch();
                    this.$emit('input', this.linkForm);
                    this.dialog = false;
                } else {
                    return false;
                }
            });
        },
    },
}) 
/**
 * 自定义字段组件
 */
Vue.component('el-field', {
    template:`
    <div class="el-field" :class="{'el-field-set': ifset}" :style="{display: ifset ? 'flex' : 'block'}">
        <div class="el-field-push" v-if="ifset">
            <draggable class="add-draggable" v-model="field" v-bind="addDraggable" :clone="addItem">
                <div v-for="(item, index) in field" class="el-field-move-item">
                    <i class="iconfont" :class="item.icon"></i>
                    <div class="title">{{item.label}}</div>
                </div>
            </draggable>
        </div>
        <el-form class="el-field-content" :class="{notset: ifset == false, set: ifset}" :label-width="labelWidth" :label-position="labelPosition" @submit.native.prevent>
            <draggable :class="{empty: list.length == 0 && ifset}" v-model="list" v-bind="draggable">
                <el-form-item v-for="(item, index) in list" :key="index" class="el-form-draggable" :class="{'el-form-set': ifset}">
                    <template slot="label">
                        <el-tooltip v-if="ifset == false" placement="top" :content="formVariable(item)">
                            <div>{{item.label}}：</div>
                        </el-tooltip>
                    </template>
                    <div class="curd-icon" v-if="ifset">
                        <i class="el-icon-copy-document" @click="copyItem(item, index)" title="复制"></i>
                        <i class="el-icon-edit" @click="setItem(item, index)" title="编辑"></i>
                        <i class="rank el-icon-rank" title="移动"></i>
                        <i class="el-icon-delete" @click="removeItem(item,index)" title="删除"></i>
                    </div>
                    <component v-if="show" v-model="item.type.value" :is="item.type.is" :key="item.type.field" :type="item.type.type"></component>
                </el-form-item>
            </draggable>
        </el-form>
        <el-dialog :visible.sync="setShow" title="字段设置" width="500px" top="30px" :close-on-click-modal="false" append-to-body>
            <el-form ref="setForm" :model="setForm" :rules="rules" :label-width="labelWidth" @submit.native.prevent>
                <el-form-item label="字段标题：" prop="label"> 
                    <el-input v-model="setForm.label" placeholder="如：内容"></el-input>
                </el-form-item>
                <el-form-item label="字段变量：" prop="field">
                    <el-input v-model="setForm.field" placeholder="如：content"></el-input>
                </el-form-item>
            </el-form>
            <span slot="footer" class="dialog-footer">
                <el-button size="small" type="primary" @click="changeItem()">确 定</el-button>
                <el-button size="small" @click="setShow = false">取 消</el-button>
            </span>
        </el-dialog>
    </div>
    `,
    props: {
        labelWidth: {
            type: String,
            default: '120px',
        },
        labelPosition: {
            type: String,
            default: 'right',
        },
        variable: {
            type: String,
            default: '',
        },
        ifset: {
            type: Boolean,
            default: true,
        },
        repeat: {
            type: Object,
            default: () => {
                return {};
            },
        },
        value: {
            type: Array,
            default: () => {
                return [];
            },
        },
    },
    data() {
        return {
            show: true,
            list: this.value,
            field: common.formType(),
            draggable: {
                handle: '.rank',
                animation: 300,
                forceFallback: true,
                group:"people"
            },
            addDraggable: {
                animation: 300,
                forceFallback: true,
                sort: false,
                group: {name: 'people', pull: 'clone', put: false},
            },
            rules: {
                label: [
                    { required: true, message: '请填写字段标题', trigger: 'blur' },
                ],
                field: [
                    { required: true, message: '请输入字段变量', trigger: 'blur' },
                ],
                type: [
                    { required: true, message: '请选择字段类型', trigger: 'blur' },
                ],
            },
            setForm: {},
            setShow: false,
            setIndex: 0,
        }
    },
    methods: {
        /**
         * 新增
         */
        addItem(item) {
            let arr         = {};
            arr.field      = common.id(6);
            arr.label      = '未命名';
            arr.type       = item;
            return JSON.parse(JSON.stringify(arr));
        },
        /**
         * 删除
         */
        removeItem(item,index) {
            let self = this;
            self.list.splice(index, 1);
            if (item.type.is == 'el-array') {
                self.show = false;
                self.$nextTick(() => {
                    self.show = true;
                })
            }
        },
        /**
         * 复制字段
         */
        copyItem(item,index) {
            let text = item.field;
            var isRTL = document.documentElement.getAttribute('dir') === 'rtl';
            var element = document.createElement('textarea');
            // 防止在ios中产生缩放效果
            element.style.fontSize = '12pt';
            // 重置盒模型
            element.style.border = '0';
            element.style.padding = '0';
            element.style.margin = '0';
            // 将元素移到屏幕外
            element.style.position = 'absolute';
            element.style[isRTL ? 'right' : 'left'] = '-9999px';
            // 移动元素到页面底部
            let yPosition = window.pageYOffset || document.documentElement.scrollTop;
            element.style.top = `${yPosition}px`;
            //设置元素只读
            element.setAttribute('readonly', '');
            element.value = text;
            document.body.appendChild(element);
            element.select();
            element.setSelectionRange(0, element.value.length);
            document.execCommand('copy');
            element.remove();
            this.$notify({showClose: true, message: '字段名称复制成功', type: 'success'});
        },
        /**
         * 设置
         */
        setItem(item, index) {
            this.setIndex = index;
            this.setForm  = JSON.parse(JSON.stringify(item));
            this.setShow  = true;
        },
        /**
         * 改变
         */
        changeItem() {
            this.$refs.setForm.validate((valid) => {
                if (valid) {
                    this.$set(this.list, this.setIndex, this.setForm);
                    this.setShow = false;
                } else {
                    return false;
                }
            });
        },
        /**
         * 表单提示变量
         */
        formVariable(item) {
            let raw = item.type.is == 'el-editor' ? '|raw' : '';
            return this.variable === '' ? item.field : '{$' + this.variable + '.' + item.field + raw +'}';
        },
    },
    watch: {
        value(val) {
            this.list = val;
        },
        list(val) {
            this.$emit('input', val);
        }
    }
})
/**
 * 参数组件
 */
Vue.component('el-parameter', {
    template:`
    <div class="el-parameter">
        <el-button type="primary" size="small" @click="pushData()">新增选项</el-button>
        <el-input v-model="keyword" v-if="search" style="width:400px" size="small" prefix-icon="el-icon-search" placeholder="请输入关键词搜索"></el-input>
        <draggable v-model="searchList" v-bind="draggable">
            <div v-for="(item, index) in searchList" :key="index" class="el-parameter-item">
                <el-input v-model="item.title" placeholder="标题" class="title"></el-input>
                <el-input v-model="item.value" placeholder="内容" class="value"></el-input>
                <el-button size="mini" v-if="rank" class="rank" type="info" icon="el-icon-rank" circle></el-button>
                <el-button size="mini" v-if="push" type="primary" icon="el-icon-plus" @click="pushData(index)" circle></el-button>
                <el-button size="mini" type="danger" icon="el-icon-close" @click="removeData(index)" circle></el-button>
            </div>
        </draggable>
    </div>
    `,
    props: {
        value: {
            type: Array,
            default: [],
        },
        rank: {
            type: Boolean,
            default: true,
        },
        push: {
            type: Boolean,
            default: true,
        },
        search: {
            type: Boolean,
            default: false,
        },
    },
    data() {
        return {
            list: this.value,
            draggable: {
                handle: '.rank',
                animation: 300,
                forceFallback: true,
            },
            keyword: '',
        }
    },
    computed: {
        searchList() {
            let list = [];
            let self = this;
            self.list.forEach(function (item, index) {
                if (item.title.indexOf(self.keyword) != -1 || item.value.indexOf(self.keyword) != -1) {
                    list.push(item)
                }
            });
            return list;
        },
    },
    created() {
        if (typeof this.list != 'object') {
            this.list = [];
        }
    },
    methods: {
        pushData(index = "") {
            let row = {title: '', value: ''};
            index === "" ? this.list.unshift(row) : this.list.splice(index + 1, 0, row);
        },
        removeData(index) {
            this.list.splice(index, 1);
        },
    },
    watch: {
        value(val) {
            this.list = val;
        },
        list(val) {
            this.$emit('input', val);
        }
    }
})
/**
 * 分类选择器
 */
Vue.component('el-catalog-select', {
    template: `
    <el-select class="el-catalog-select" v-model="content" placeholder="请选择分类" filterable>
        <el-option label="未选择" :value="0"></el-option>
        <el-option v-for="(item, index) in catalogList" :key="index" :label="item.title" :value="item.id">
            <span v-html="item.treeString"></span>
            {{ item.title }}
        </el-option>
    </el-select>
    `,
    props: {
        value: {
            type: Number,
        },
    },
    data() {
        return {
            catalogList: [],
            catalogUrl: "api/catalog",
            content: this.value,
        }
    },
    created() {
        let self = this;
        request.post(self.catalogUrl, {theme: theme}, function(res) {
            self.catalogList = tree.convertString(res.data);
        });
    },
    watch: {
        value(val) {
            this.content = val;
        },
        content(val) {
            this.$emit('input', val);
        },
    }
})
/**
 * 编辑器默认是textarea
 */
Vue.component('el-editor', {
    template: `
        <div class="el-editor">
            <el-input type="textarea" v-model="content" :rows="10"></el-input>
        </div>
    `,
    props: {
        value: {
            type: String,
            default: "",
        },
    },
    data() {
        return {
            content: this.value,
        }
    },
    watch: {
        value(val) {
            this.content = val;
        },
        content(val) {
            this.$emit('input', val);
        },
    }
})
/**
 * form里显示html代码
 */
Vue.component('el-html', {
    template: `<div class="el-html" v-html="content"></div>`,
    props: {
        value: {
            type: String,
            default: "",
        },
    },
    data() {
        return {
            content: this.value,
        }
    },
    watch: {
        value(val) {
            this.content = val;
        },
    }
})